@@ -24,7 +24,7 @@
 | 
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: context_menus.c 1827 2007-12-13 22:55:50Z phantom_sf $
+|  $Id: context_menus.c 2027 2008-07-05 03:14:06Z jcsjcs $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -319,6 +319,8 @@ static void eject_ipod (GtkMenuItem *mi, gpointer data)
     g_return_if_fail (eitdb->itdb_imported == TRUE);
 
     gp_eject_ipod (itdb);
+    /* Set the coverart display based on the selected playlist */
+    coverart_display_update(TRUE);
 }
 
 
@@ -557,7 +559,7 @@ static GtkWidget *add_copy_selected_to_target_itdb (GtkWidget *menu,
     GList *db;
     struct itdbs_head *itdbs_head;
     iTunesDB *itdb;
-    gchar *stock_id = NULL;
+    const gchar *stock_id = NULL;
     Playlist *pl;
 
   
@@ -1158,15 +1160,7 @@ cad_context_menu_init(void)
     
      if (selected_tracks) g_list_free (selected_tracks);
     	selected_tracks = g_list_copy (coverart_get_displayed_tracks());
-    /*
-    int i;
-    for (i = 0; i < g_list_length(selected_tracks); ++i)
-    {
-    	Track *track;
-    	track = g_list_nth_data (selected_tracks, i);
-    	printf ("context_menu_init - Artist:%s  Album:%s  Title:%s\n", track->artist, track->album, track->title);
-    }
-    */
+    	
     if(selected_tracks)
 			create_context_menu (CM_CAD);
 }
@@ -1213,10 +1207,8 @@ pm_context_menu_init(void)
     entry_inst = -1;
     selected_playlist = pm_get_selected_playlist();
     
-    /* Dont allow context menu to display if the playlist is the photo one */
-    if (gphoto_is_photo_playlist (selected_playlist)) return;
-        
     active_itdb = gp_get_selected_itdb ();
+
     if(selected_playlist)
     {
 	selected_tracks = g_list_copy (selected_playlist->members);
