@@ -1,4 +1,4 @@
-/* Time-stamp: <2007-04-23 22:24:05 jcs>
+/* Time-stamp: <2008-12-07 19:10:37 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: display.c 1053 2007-04-23 15:33:31Z jcsjcs $
+|  $Id: display.c 2156 2008-12-07 11:39:42Z jcsjcs $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -34,17 +34,20 @@
 #include "details.h"
 #include "display_private.h"
 #include "info.h"
+#include "infodlg.h"
 #include "ipod_init.h"
 #include "file_convert.h"
 #include "misc.h"
 #include "misc_track.h"
 #include "prefs.h"
-#include "prefs_window.h"
+#include "sort_window.h"
 #include "repository.h"
 #include "syncdir.h"
 #include "tools.h"
 #include <stdlib.h>
 #include <string.h>
+#include <cairo.h>
+#include <glib/gprintf.h>
 
 
 GtkWidget *gtkpod_window = NULL;
@@ -75,12 +78,19 @@ void display_create (void)
 
     tm_create_treeview ();
     st_create_tabs ();
-    st_set_default_sizes ();
     pm_create_treeview ();
+
+    /* initialize sorting */
+    tm_sort (prefs_get_int("tm_sortcol"), prefs_get_int("tm_sort"));
+
+    st_set_default_sizes ();
+
     /* Hide/Show the toolbar */
     display_show_hide_toolbar ();
     /* Hide/Show tooltips */
     display_show_hide_tooltips ();
+    /* Hide/Show searchbar */
+    display_show_hide_searchbar ();
     /* change standard g_print () handler */
     g_set_print_handler ((GPrintFunc)gtkpod_warning);
 
@@ -98,7 +108,8 @@ void display_create (void)
     /* CAREFUL: must be done after calling ..._space_statusbar_init() */
     display_set_info_window_menu ();
     /* check if info window should be opened */
-    if (prefs_get_int("info_window"))  info_open_window ();
+    if (prefs_get_int("info_window"))
+		open_info_dialog ();
 }
 
 /* redisplay the entire display (playlists, sort tabs, track view) and
@@ -144,7 +155,7 @@ void display_cleanup (void)
 }
 
 
-/* make sure only suitable delete menu items are available */
+/* make sure only suitable menu items are available */
 void display_adjust_menus (void)
 {
     GtkWidget *delete;
@@ -162,6 +173,13 @@ void display_adjust_menus (void)
     edit3 = gtkpod_xml_get_widget (main_window_xml, "create_playlists_menu");
     edit4 = gtkpod_xml_get_widget (main_window_xml, "randomize_current_playlist_menu");
     edit5 = gtkpod_xml_get_widget (main_window_xml, "save_track_order_menu");
+
+#if 0
+    edit6 = gtkpod_xml_get_widget (main_window_xml, "add_files1");
+    edit7 = gtkpod_xml_get_widget (main_window_xml, "add_directory1");
+    edit8 = gtkpod_xml_get_widget (main_window_xml, "add_playlist1");
+#endif
+
     dtfpl = gtkpod_xml_get_widget (main_window_xml,
 				   "delete_selected_tracks_from_playlist");
     dtfip = gtkpod_xml_get_widget (main_window_xml,
@@ -191,6 +209,13 @@ void display_adjust_menus (void)
     espl = gtkpod_xml_get_widget (main_window_xml,
 				  "edit_smart_playlist");
 
+#if 0
+    tb_edit1 = gtkpod_xml_get_widget (main_window_xml, "add_files_button");
+    tb_edit2 = gtkpod_xml_get_widget (main_window_xml, "add_dirs_button");
+    tb_edit3 = gtkpod_xml_get_widget (main_window_xml, "add_PL_button");
+    tb_edit4 = gtkpod_xml_get_widget (main_window_xml, "new_PL_button");
+#endif
+
     pl = pm_get_selected_playlist ();
 
     if (pl == NULL)
@@ -201,6 +226,7 @@ void display_adjust_menus (void)
 	gtk_widget_set_sensitive (edit3, FALSE);
 	gtk_widget_set_sensitive (edit4, FALSE);
 	gtk_widget_set_sensitive (edit5, FALSE);
+	gtk_widget_set_sensitive (espl, FALSE);
     }
     else
     {
@@ -355,6 +381,34 @@ void display_show_hide_toolbar (void)
     }
 }
 
+/* make the search bar visible or hide it depending on the value set in
+ * the prefs
+ */
+void display_show_hide_searchbar (void)
+{
+	GtkWidget *upbutton = gtkpod_xml_get_widget (main_window_xml, "searchbar_up_button");
+	GtkWidget *searchbar = gtkpod_xml_get_widget (main_window_xml, "searchbar_hpanel");
+	GtkCheckMenuItem *mi = GTK_CHECK_MENU_ITEM (gtkpod_xml_get_widget (main_window_xml, "filterbar_menu"));
+
+	g_return_if_fail (upbutton);
+	g_return_if_fail (searchbar);
+	g_return_if_fail (mi);
+		
+	if (prefs_get_int ("display_search_entry"))
+	{
+		gtk_widget_show_all (searchbar);
+		gtk_widget_hide (upbutton);
+		gtk_check_menu_item_set_active (mi, TRUE);
+	}
+	else
+	{		
+		gtk_widget_hide_all (searchbar);
+		gtk_widget_show (upbutton);
+		gtk_widget_set_sensitive (upbutton, TRUE);
+		gtk_check_menu_item_set_active (mi, FALSE);
+	}
+}
+
 /* Adjust the menu item status according on the value set in the
    prefs. */
 void display_set_info_window_menu (void)
@@ -412,8 +466,6 @@ void display_show_hide_tooltips (void)
     /* Show/Hide tooltips of the special sorttabs */
     st_show_hide_tooltips ();
     /* Show/Hide tooltips of the prefs window */
-    prefs_window_show_hide_tooltips ();
-    /* Show/Hide tooltips of the prefs window */
     sort_window_show_hide_tooltips ();
 }
 
@@ -436,13 +488,126 @@ void display_update_default_sizes (void)
     }
     tm_update_default_sizes ();
     st_update_default_sizes ();
-    prefs_window_update_default_sizes ();
-    info_update_default_sizes ();
+    info_dialog_update_default_sizes ();
     details_update_default_sizes ();
     file_convert_update_default_sizes ();
 }
 
+/**
+ * on_drawing_area_exposed:
+ *
+ * Callback for the drawing area. When the drwaing area is covered,
+ * resized, changed etc.. This will be called the draw() function is then
+ * called from this and the cairo redrawing takes place.
+ * 
+ * @draw_area: drawing area where al the cairo drawing takes place 
+ * @event: gdk expose event
+ * 
+ * Returns:
+ * boolean indicating whether other handlers should be run.
+ */
+static gboolean on_coverart_preview_dialog_exposed (GtkWidget *drawarea, GdkEventExpose *event, gpointer data)
+{
+	GdkPixbuf *image = data;
+	
+	/* Draw the image using cairo */
+		cairo_t *cairo_context;
+		
+		/* get a cairo_t */
+		cairo_context = gdk_cairo_create (drawarea->window);
+		/* set a clip region for the expose event */
+		cairo_rectangle (cairo_context,
+				event->area.x, event->area.y,
+				event->area.width, event->area.height);
+		cairo_clip (cairo_context);
+	
+		gdk_cairo_set_source_pixbuf (
+				cairo_context,
+				image,
+				0,
+				0);
+		cairo_paint (cairo_context);
+		
+		cairo_destroy (cairo_context);
+		return FALSE;
+}
 
+/**
+ * display_image_dialog
+ * 
+ * @GdkPixbuf: image
+ * 
+ * function to load a transient dialog displaying the provided image at either
+ * it maximum size or the size of the screen (whichever is smallest).
+ * 
+ */
+void display_image_dialog (GdkPixbuf *image)
+{
+	g_return_if_fail (image);
+	
+	GladeXML *preview_xml;
+	GtkWidget *dialog;
+	GtkWidget *drawarea;
+	GtkWidget *res_label;
+	GdkPixbuf *scaled = NULL;
+	gchar *text;
+		
+	preview_xml = gtkpod_xml_new (xml_file, "coverart_preview_dialog");
+	dialog = gtkpod_xml_get_widget (preview_xml, "coverart_preview_dialog");
+	drawarea = gtkpod_xml_get_widget (preview_xml, "coverart_preview_dialog_drawarea");
+	res_label = gtkpod_xml_get_widget (preview_xml, "coverart_preview_dialog_res_lbl");
+	g_return_if_fail (dialog);
+	g_return_if_fail (drawarea);
+	g_return_if_fail (res_label);
+		
+	/* Set the dialog parent */
+	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (gtkpod_window));
+			
+	gint pixheight = gdk_pixbuf_get_height (image);
+	gint pixwidth = gdk_pixbuf_get_width (image);
+		
+	/* Set the resolution in the label */
+	text = g_markup_printf_escaped (_("<b>Image Dimensions: %d x %d</b>"), pixwidth, pixheight);
+	gtk_label_set_markup (GTK_LABEL (res_label), text);
+	g_free (text);
+	
+	gint scrheight = gdk_screen_height() - 100;
+	gint scrwidth = gdk_screen_width() - 100;
+		
+	gdouble ratio = (gdouble) pixwidth / (gdouble) pixheight;
+	if (pixwidth > scrwidth)
+	{
+		pixwidth = scrwidth;
+		pixheight = pixwidth / ratio;
+	}
+		
+	if (pixheight > scrheight)
+	{
+		pixheight = scrheight;
+		pixwidth = pixheight * ratio;
+	}
+	
+	scaled = gdk_pixbuf_scale_simple (
+					image, 
+					pixwidth, 
+					pixheight, 
+					GDK_INTERP_BILINEAR);
+
+	/* Set the draw area's minimum size */
+	gtk_widget_set_size_request  (drawarea, pixwidth, pixheight);
+	g_signal_connect (G_OBJECT (drawarea), "expose_event",  G_CALLBACK (on_coverart_preview_dialog_exposed), scaled);
+	
+	/* Display the dialog and block everything else until the
+	 * dialog is closed.
+	 */
+	gtk_widget_show_all (dialog);
+	gtk_dialog_run (GTK_DIALOG(dialog));
+			
+	/* Destroy the dialog as no longer required */
+	
+	g_object_unref (scaled);
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+}
 
 /* Utility function: returns a copy of the tracks currently
    selected. This means:
@@ -625,7 +790,8 @@ void display_install_autoscroll_row_timeout (GtkWidget *widget)
 */
 
 
-void on_edit_details_selected_playlist (GtkMenuItem     *menuitem,
+G_MODULE_EXPORT void
+on_edit_details_selected_playlist (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
     Playlist *pl = pm_get_selected_playlist ();
@@ -640,7 +806,8 @@ void on_edit_details_selected_playlist (GtkMenuItem     *menuitem,
     }
 }
 
-void on_edit_details_selected_tab_entry (GtkMenuItem     *menuitem,
+G_MODULE_EXPORT void
+on_edit_details_selected_tab_entry (GtkMenuItem     *menuitem,
 					 gpointer         user_data)
 {
     gint inst = get_sort_tab_number (
@@ -658,7 +825,8 @@ void on_edit_details_selected_tab_entry (GtkMenuItem     *menuitem,
     }
 }
 
-void on_edit_details_selected_tracks (GtkMenuItem     *menuitem,
+G_MODULE_EXPORT void
+on_edit_details_selected_tracks (GtkMenuItem     *menuitem,
 				      gpointer         user_data)
 {
     GList *tracks = tm_get_selected_tracks ();
@@ -676,7 +844,7 @@ void on_edit_details_selected_tracks (GtkMenuItem     *menuitem,
 
 
 /* callback for "add new playlist" button */
-void
+G_MODULE_EXPORT void
 on_new_playlist_button                 (GtkButton       *button,
 					gpointer         user_data)
 {
@@ -694,7 +862,7 @@ on_new_playlist_button                 (GtkButton       *button,
 
 
 /* callback for "add new playlist" menu */
-void
+G_MODULE_EXPORT void
 on_new_playlist1_activate              (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
@@ -710,14 +878,7 @@ on_new_playlist1_activate              (GtkMenuItem     *menuitem,
     }
 }
 
-void
-on_edit_preferences_activate          (GtkMenuItem     *menuitem,
-					gpointer         user_data)
-{
-    if(!widgets_blocked)  prefs_window_create (-1);
-}
-
-void
+G_MODULE_EXPORT void
 on_edit_repository_options_activate          (GtkMenuItem     *menuitem,
 					      gpointer         user_data)
 {
@@ -733,7 +894,7 @@ on_edit_repository_options_activate          (GtkMenuItem     *menuitem,
     }
 }
 
-void
+G_MODULE_EXPORT void
 on_load_ipods_mi           (GtkMenuItem     *menuitem,
 			    gpointer         user_data)
 {
@@ -741,7 +902,7 @@ on_load_ipods_mi           (GtkMenuItem     *menuitem,
 }
 
 
-void
+G_MODULE_EXPORT void
 on_load_ipods_clicked               (GtkButton       *button,
 				     gpointer         user_data)
 {
@@ -749,7 +910,7 @@ on_load_ipods_clicked               (GtkButton       *button,
 }
 
 
-void
+G_MODULE_EXPORT void
 on_save_changes_mi           (GtkMenuItem     *menuitem,
 			      gpointer         user_data)
 {
@@ -758,7 +919,7 @@ on_save_changes_mi           (GtkMenuItem     *menuitem,
 
 
 
-void
+G_MODULE_EXPORT void
 on_save_changes_clicked               (GtkButton       *button,
 				     gpointer         user_data)
 {
@@ -766,7 +927,7 @@ on_save_changes_clicked               (GtkButton       *button,
 }
 
 
-
+G_MODULE_EXPORT 
 void on_edit_smart_playlist (GtkMenuItem *mi,
 			     gpointer data)
 {
@@ -831,13 +992,14 @@ static void delete_selected_playlist (DeleteAction deleteaction)
     }
 }
 
-void on_delete_selected_tracks_from_database (GtkMenuItem *mi,
+G_MODULE_EXPORT void
+on_delete_selected_tracks_from_database (GtkMenuItem *mi,
 					      gpointer data)
 {
     delete_selected_tracks (DELETE_ACTION_DATABASE);
 }
 
-void
+G_MODULE_EXPORT void
 on_delete_selected_playlist_including_tracks_from_harddisk (GtkMenuItem *mi,
 							    gpointer data)
 {
@@ -845,7 +1007,8 @@ on_delete_selected_playlist_including_tracks_from_harddisk (GtkMenuItem *mi,
 }
 
 
-void on_delete_selected_entry_from_database (GtkMenuItem *mi,
+G_MODULE_EXPORT void
+on_delete_selected_entry_from_database (GtkMenuItem *mi,
 					     gpointer data)
 {
     delete_selected_entry (DELETE_ACTION_DATABASE, 
@@ -853,7 +1016,8 @@ void on_delete_selected_entry_from_database (GtkMenuItem *mi,
 }
 
 
-void on_delete_selected_entry_from_ipod (GtkMenuItem *mi,
+G_MODULE_EXPORT void
+on_delete_selected_entry_from_ipod (GtkMenuItem *mi,
 					 gpointer data)
 {
     delete_selected_entry (DELETE_ACTION_IPOD,
@@ -861,21 +1025,24 @@ void on_delete_selected_entry_from_ipod (GtkMenuItem *mi,
 }
 
 
-void on_delete_selected_tracks_from_playlist (GtkMenuItem *mi,
+G_MODULE_EXPORT void
+on_delete_selected_tracks_from_playlist (GtkMenuItem *mi,
 					      gpointer data)
 {
     delete_selected_tracks (DELETE_ACTION_PLAYLIST);
 }
 
 
-void on_delete_selected_tracks_from_harddisk (GtkMenuItem *mi,
+G_MODULE_EXPORT void
+on_delete_selected_tracks_from_harddisk (GtkMenuItem *mi,
 					      gpointer data)
 {
     delete_selected_tracks (DELETE_ACTION_LOCAL);
 }
 
 
-void on_delete_selected_entry_from_harddisk (GtkMenuItem *mi,
+G_MODULE_EXPORT void
+on_delete_selected_entry_from_harddisk (GtkMenuItem *mi,
 					     gpointer data)
 {
     delete_selected_entry (DELETE_ACTION_LOCAL,
@@ -883,21 +1050,23 @@ void on_delete_selected_entry_from_harddisk (GtkMenuItem *mi,
 }
 
 
-void on_delete_selected_tracks_from_ipod (GtkMenuItem *mi,
+G_MODULE_EXPORT void
+on_delete_selected_tracks_from_ipod (GtkMenuItem *mi,
 					  gpointer data)
 {
     delete_selected_tracks (DELETE_ACTION_IPOD);
 }
 
 
-void on_delete_selected_playlist (GtkMenuItem *mi,
+G_MODULE_EXPORT void
+on_delete_selected_playlist (GtkMenuItem *mi,
 				  gpointer data)
 {
     delete_selected_playlist (DELETE_ACTION_PLAYLIST);
 }
 
 
-void
+G_MODULE_EXPORT void
 on_delete_selected_playlist_including_tracks_from_database (GtkMenuItem *mi,
 							    gpointer data)
 {
@@ -905,7 +1074,8 @@ on_delete_selected_playlist_including_tracks_from_database (GtkMenuItem *mi,
 }
 
 
-void on_delete_selected_entry_from_playlist (GtkMenuItem *mi,
+G_MODULE_EXPORT void
+on_delete_selected_entry_from_playlist (GtkMenuItem *mi,
 					     gpointer data)
 {
     delete_selected_entry (DELETE_ACTION_PLAYLIST,
@@ -913,7 +1083,7 @@ void on_delete_selected_entry_from_playlist (GtkMenuItem *mi,
 }
 
 
-void
+G_MODULE_EXPORT void
 on_delete_selected_playlist_including_tracks_from_ipod (GtkMenuItem *mi,
 							gpointer data)
 {
@@ -921,7 +1091,7 @@ on_delete_selected_playlist_including_tracks_from_ipod (GtkMenuItem *mi,
 }
 
 
-void
+G_MODULE_EXPORT void
 on_ipod_directories_menu               (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
@@ -959,7 +1129,7 @@ on_ipod_directories_menu               (GtkMenuItem     *menuitem,
     }
 }
 
-void
+G_MODULE_EXPORT void
 on_check_ipod_files_activate           (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
@@ -998,7 +1168,7 @@ on_check_ipod_files_activate           (GtkMenuItem     *menuitem,
 }
 
 
-void
+G_MODULE_EXPORT void
 on_update_playlist_activate (GtkMenuItem     *menuitem,
 			     gpointer         user_data)
 {
@@ -1006,7 +1176,7 @@ on_update_playlist_activate (GtkMenuItem     *menuitem,
 }
 
 /* update tracks in tab entry */
-void
+G_MODULE_EXPORT void
 on_update_tab_entry_activate        (GtkMenuItem     *menuitem,
 				     gpointer         user_data)
 {
@@ -1016,15 +1186,22 @@ on_update_tab_entry_activate        (GtkMenuItem     *menuitem,
     if (inst != -1) gp_do_selected_entry (update_tracks, inst);
 }
 
-void
+G_MODULE_EXPORT void
 on_update_tracks_activate            (GtkMenuItem     *menuitem,
 				     gpointer         user_data)
 {
     gp_do_selected_tracks (update_tracks);
 }
 
+G_MODULE_EXPORT void
+on_mserv_from_file_playlist_menu_activate
+                                        (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+    gp_do_selected_playlist (mserv_from_file_tracks);
+}
 
-void
+G_MODULE_EXPORT void
 on_mserv_from_file_tracks_menu_activate
                                         (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
@@ -1033,7 +1210,7 @@ on_mserv_from_file_tracks_menu_activate
 }
 
 
-void
+G_MODULE_EXPORT void
 on_mserv_from_file_entry_menu_activate (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
@@ -1044,7 +1221,7 @@ on_mserv_from_file_entry_menu_activate (GtkMenuItem     *menuitem,
 }
 
 
-void
+G_MODULE_EXPORT void
 on_sync_playlist_activate (GtkMenuItem     *menuitem,
 			     gpointer         user_data)
 {
@@ -1065,7 +1242,7 @@ on_sync_playlist_activate (GtkMenuItem     *menuitem,
     }
 }
 
-void
+G_MODULE_EXPORT void
 on_save_track_order1_activate           (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
@@ -1073,7 +1250,7 @@ on_save_track_order1_activate           (GtkMenuItem     *menuitem,
 }
 
 
-void
+G_MODULE_EXPORT void
 on_toolbar_menu_activate               (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
@@ -1083,7 +1260,17 @@ on_toolbar_menu_activate               (GtkMenuItem     *menuitem,
 }
 
 
-void
+G_MODULE_EXPORT void
+on_filterbar_menu_activate               (GtkMenuItem     *menuitem,
+					  gpointer         user_data)
+{
+    prefs_set_int("display_search_entry",
+	gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (menuitem)));
+    display_show_hide_searchbar ();
+}
+
+
+G_MODULE_EXPORT void
 on_more_sort_tabs_activate             (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
@@ -1092,7 +1279,7 @@ on_more_sort_tabs_activate             (GtkMenuItem     *menuitem,
 }
 
 
-void
+G_MODULE_EXPORT void
 on_less_sort_tabs_activate             (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
@@ -1100,7 +1287,7 @@ on_less_sort_tabs_activate             (GtkMenuItem     *menuitem,
     st_show_visible();
 }
 
-void
+G_MODULE_EXPORT void
 on_export_playlist_activate  (GtkMenuItem     *menuitem,
 			      gpointer         user_data)
 {
@@ -1117,7 +1304,7 @@ on_export_playlist_activate  (GtkMenuItem     *menuitem,
 }
 
 
-void
+G_MODULE_EXPORT void
 on_export_tab_entry_activate (GtkMenuItem     *menuitem,
 			      gpointer         user_data)
 {
@@ -1138,7 +1325,7 @@ on_export_tab_entry_activate (GtkMenuItem     *menuitem,
 }
 
 
-void
+G_MODULE_EXPORT void
 on_export_tracks_activate     (GtkMenuItem     *menuitem,
 			      gpointer         user_data)
 {
@@ -1156,7 +1343,7 @@ on_export_tracks_activate     (GtkMenuItem     *menuitem,
 }
 
 
-void
+G_MODULE_EXPORT void
 on_playlist_file_playlist_activate     (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
@@ -1173,7 +1360,7 @@ on_playlist_file_playlist_activate     (GtkMenuItem     *menuitem,
 }
 
 
-void
+G_MODULE_EXPORT void
 on_playlist_file_tab_entry_activate    (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
@@ -1194,7 +1381,7 @@ on_playlist_file_tab_entry_activate    (GtkMenuItem     *menuitem,
 }
 
 
-void
+G_MODULE_EXPORT void
 on_playlist_file_tracks_activate       (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
@@ -1211,7 +1398,7 @@ on_playlist_file_tracks_activate       (GtkMenuItem     *menuitem,
     }
 }
 
-void
+G_MODULE_EXPORT void
 on_play_playlist_activate              (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
@@ -1227,7 +1414,7 @@ on_play_playlist_activate              (GtkMenuItem     *menuitem,
 }
 
 
-void
+G_MODULE_EXPORT void
 on_play_tab_entry_activate             (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
@@ -1248,7 +1435,7 @@ on_play_tab_entry_activate             (GtkMenuItem     *menuitem,
 }
 
 
-void
+G_MODULE_EXPORT void
 on_play_tracks_activate                 (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
@@ -1266,7 +1453,7 @@ on_play_tracks_activate                 (GtkMenuItem     *menuitem,
 }
 
 
-void
+G_MODULE_EXPORT void
 on_enqueue_playlist_activate           (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
@@ -1282,7 +1469,7 @@ on_enqueue_playlist_activate           (GtkMenuItem     *menuitem,
 }
 
 
-void
+G_MODULE_EXPORT void
 on_enqueue_tab_entry_activate          (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
@@ -1303,7 +1490,7 @@ on_enqueue_tab_entry_activate          (GtkMenuItem     *menuitem,
 }
 
 
-void
+G_MODULE_EXPORT void
 on_enqueue_tracks_activate              (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
@@ -1321,14 +1508,14 @@ on_enqueue_tracks_activate              (GtkMenuItem     *menuitem,
 }
 
 
-void
+G_MODULE_EXPORT void
 on_arrange_sort_tabs_activate          (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
     st_arrange_visible_sort_tabs ();
 }
 
-void
+G_MODULE_EXPORT void
 on_tooltips_menu_activate              (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
@@ -1338,21 +1525,21 @@ on_tooltips_menu_activate              (GtkMenuItem     *menuitem,
     
 }
 
-void
+G_MODULE_EXPORT void
 on_pl_containing_displayed_tracks_activate (GtkMenuItem     *menuitem,
 					    gpointer         user_data)
 {
     generate_displayed_playlist ();
 }
 
-void
+G_MODULE_EXPORT void
 on_pl_containing_selected_tracks_activate (GtkMenuItem     *menuitem,
 					    gpointer         user_data)
 {
     generate_selected_playlist ();
 }
 
-void
+G_MODULE_EXPORT void
 on_pl_for_each_artist_activate         (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
@@ -1368,7 +1555,7 @@ on_pl_for_each_artist_activate         (GtkMenuItem     *menuitem,
     }
 }
 
-void
+G_MODULE_EXPORT void
 on_pl_for_each_album_activate         (GtkMenuItem     *menuitem,
 				       gpointer         user_data)
 {
@@ -1384,7 +1571,7 @@ on_pl_for_each_album_activate         (GtkMenuItem     *menuitem,
     }
 }
 
-void
+G_MODULE_EXPORT void
 on_pl_for_each_genre_activate         (GtkMenuItem     *menuitem,
 				       gpointer         user_data)
 {
@@ -1400,7 +1587,7 @@ on_pl_for_each_genre_activate         (GtkMenuItem     *menuitem,
     }
 }
 
-void
+G_MODULE_EXPORT void
 on_pl_for_each_composer_activate         (GtkMenuItem     *menuitem,
 					  gpointer         user_data)
 {
@@ -1417,7 +1604,7 @@ on_pl_for_each_composer_activate         (GtkMenuItem     *menuitem,
 }
 
 
-void
+G_MODULE_EXPORT void
 on_pl_for_each_year_activate           (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
@@ -1434,7 +1621,7 @@ on_pl_for_each_year_activate           (GtkMenuItem     *menuitem,
 }
 
 
-void
+G_MODULE_EXPORT void
 on_most_listened_tracks1_activate       (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
@@ -1451,7 +1638,7 @@ on_most_listened_tracks1_activate       (GtkMenuItem     *menuitem,
 }
 
 
-void
+G_MODULE_EXPORT void
 on_all_tracks_never_listened_to1_activate
 					(GtkMenuItem     *menuitem,
 					gpointer         user_data)
@@ -1468,7 +1655,7 @@ on_all_tracks_never_listened_to1_activate
     }
 }
 
-void
+G_MODULE_EXPORT void
 on_most_rated_tracks_playlist_s1_activate
 					(GtkMenuItem     *menuitem,
 					gpointer         user_data)
@@ -1486,7 +1673,7 @@ on_most_rated_tracks_playlist_s1_activate
 }
 
 
-void
+G_MODULE_EXPORT void
 on_most_recent_played_tracks_activate   (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
@@ -1502,7 +1689,7 @@ on_most_recent_played_tracks_activate   (GtkMenuItem     *menuitem,
     }
 }
 
-void
+G_MODULE_EXPORT void
 on_played_since_last_time1_activate    (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
@@ -1518,14 +1705,14 @@ on_played_since_last_time1_activate    (GtkMenuItem     *menuitem,
     }
 }
 
-void
+G_MODULE_EXPORT void
 on_sorting_activate                    (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
     sort_window_create ();
 }
 
-void
+G_MODULE_EXPORT void
 on_normalize_selected_playlist_activate
 					(GtkMenuItem     *menuitem,
 					gpointer         user_data)
@@ -1538,7 +1725,7 @@ on_normalize_selected_playlist_activate
 }
 
 
-void
+G_MODULE_EXPORT void
 on_normalize_selected_tab_entry_activate
 					(GtkMenuItem     *menuitem,
 					gpointer         user_data)
@@ -1563,7 +1750,7 @@ on_normalize_selected_tab_entry_activate
 }
 
 
-void
+G_MODULE_EXPORT void
 on_normalize_selected_tracks_activate   (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
@@ -1573,7 +1760,7 @@ on_normalize_selected_tracks_activate   (GtkMenuItem     *menuitem,
 }
 
 
-void
+G_MODULE_EXPORT void
 on_normalize_displayed_tracks_activate  (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
@@ -1583,7 +1770,7 @@ on_normalize_displayed_tracks_activate  (GtkMenuItem     *menuitem,
 }
 
 
-void
+G_MODULE_EXPORT void
 on_normalize_all_tracks                (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
@@ -1602,7 +1789,7 @@ on_normalize_all_tracks                (GtkMenuItem     *menuitem,
 }
 
 
-void
+G_MODULE_EXPORT void
 on_normalize_newly_added_tracks        (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
@@ -1619,16 +1806,17 @@ on_normalize_newly_added_tracks        (GtkMenuItem     *menuitem,
 }
 
 
-void
+G_MODULE_EXPORT void
 on_info_window1_activate               (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
     if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (menuitem)))
-	 info_open_window ();
-    else info_close_window ();
+		open_info_dialog ();
+    else
+		close_info_dialog ();
 }
 
-void
+G_MODULE_EXPORT void
 on_conversion_log1_activate               (GtkMenuItem     *menuitem,
 					   gpointer         user_data)
 {
@@ -1644,7 +1832,7 @@ on_conversion_log1_activate               (GtkMenuItem     *menuitem,
 }
 
 
-void
+G_MODULE_EXPORT void
 on_sync_all_activate                   (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
@@ -1665,7 +1853,7 @@ on_sync_all_activate                   (GtkMenuItem     *menuitem,
 }
 
 
-void
+G_MODULE_EXPORT void
 on_sync_calendar_activate              (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
@@ -1681,7 +1869,7 @@ on_sync_calendar_activate              (GtkMenuItem     *menuitem,
 }
 
 
-void
+G_MODULE_EXPORT void
 on_sync_contacts_activate              (GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
@@ -1696,8 +1884,7 @@ on_sync_contacts_activate              (GtkMenuItem     *menuitem,
     }
 }
 
-
-void
+G_MODULE_EXPORT void
 on_sync_notes_activate                 (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
@@ -1711,8 +1898,7 @@ on_sync_notes_activate                 (GtkMenuItem     *menuitem,
 	message_sb_no_ipod_itdb_selected ();
     }
 }
-
-void
+G_MODULE_EXPORT void
 on_all_tracks_not_listed_in_any_playlist1_activate
                                         (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
@@ -1729,8 +1915,7 @@ on_all_tracks_not_listed_in_any_playlist1_activate
     }
 }
 
-
-void
+G_MODULE_EXPORT void
 on_random_playlist_activate            (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
@@ -1746,15 +1931,13 @@ on_random_playlist_activate            (GtkMenuItem     *menuitem,
     }
 }
 
-
-void
+G_MODULE_EXPORT void
 on_randomize_current_playlist_activate (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
     randomize_current_playlist();
 }
-
-void
+G_MODULE_EXPORT void
 on_pl_for_each_rating_activate         (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
