@@ -23,7 +23,7 @@
 | 
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: display_itdb.h 1590 2007-06-25 20:49:55Z phantom_sf $
+|  $Id: display_itdb.h 2148 2008-11-08 08:02:15Z jcsjcs $
 */
 
 #ifndef __DISPLAY_ITDB_H__
@@ -53,8 +53,10 @@ typedef struct
 				      (only for GP_ITDP_TYPE_IPOD)         */
     gboolean offline;              /* offline mode?                        */
     gboolean data_changed;         /* data changed since import?           */
+    gboolean photo_data_changed;	/* photo data changed since import?		*/
     gboolean itdb_imported;        /* has in iTunesDB been imported?       */
     gboolean ipod_ejected;         /* if iPod was ejected                  */
+    PhotoDB *photodb;            /* Photo DB reference used if the ipod supports photos */
 } ExtraiTunesDBData;
 
 typedef struct
@@ -85,6 +87,8 @@ typedef struct
 			       original itdb                               */
   guint64 local_track_dbid; /* when using DND from local to iPod:
 			       original track                              */
+  gchar   *lyrics;          /* Lyrics information as read from file or as
+			       updated in the program                      */
 } ExtraTrackData;
 
 /* types for iTunesDB */
