@@ -22,7 +22,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: display_spl.c 1057 2007-04-27 16:12:39Z jcsjcs $
+|  $Id: display_spl.c 1872 2007-12-29 18:35:12Z Sikon $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -91,7 +91,7 @@ static const ComboEntry splfield_comboentries[] =
     { ITDB_SPLFIELD_DATE_MODIFIED,  N_("Date modified") },
     { ITDB_SPLFIELD_TRACKNUMBER,    N_("Track number") },
     { ITDB_SPLFIELD_SIZE,           N_("Size") },
-    { ITDB_SPLFIELD_TIME,           N_("Time") },
+    { ITDB_SPLFIELD_TIME,           N_("Play time") },
     { ITDB_SPLFIELD_COMMENT,        N_("Comment") },
     { ITDB_SPLFIELD_DATE_ADDED,     N_("Date added") },
     { ITDB_SPLFIELD_COMPOSER,       N_("Composer") },
@@ -233,6 +233,13 @@ enum entrytype
     spl_ET_STRING,
 };
 
+enum matchmode
+{
+	spl_MATCH_ANY = 0,
+	spl_MATCH_ALL,
+	spl_MATCH_IGNORE
+};
+
 static const gchar *entry_get_string (gchar *str, Itdb_SPLRule *splr,
 				      enum entrytype et);
 
@@ -336,62 +343,6 @@ static void spl_set_combobox (GtkComboBox *cb,
 
 }
 
-
-/* Callbacks */
-static void spl_all_radio_toggled (GtkToggleButton *togglebutton,
-				   GtkWidget *spl_window)
-{
-    Playlist *spl;
-
-    g_return_if_fail (spl_window);
-    spl =  g_object_get_data (G_OBJECT (spl_window), "spl_work");
-    g_return_if_fail (spl);
-    if (gtk_toggle_button_get_active (togglebutton))
-    {
-	GtkWidget *frame = gtkpod_xml_get_widget (spl_window_xml, "spl_rules_frame");
-	g_return_if_fail (frame);
-	gtk_widget_set_sensitive (frame, TRUE);
-	spl->splpref.checkrules = TRUE;
-	spl->splrules.match_operator = ITDB_SPLMATCH_AND;
-    }
-}
-
-static void spl_any_radio_toggled (GtkToggleButton *togglebutton,
-				   GtkWidget *spl_window)
-{
-    Playlist *spl;
-
-    g_return_if_fail (spl_window);
-    spl =  g_object_get_data (G_OBJECT (spl_window), "spl_work");
-    g_return_if_fail (spl);
-    if (gtk_toggle_button_get_active (togglebutton))
-    {
-	GtkWidget *frame = gtkpod_xml_get_widget (spl_window_xml, "spl_rules_frame");
-	g_return_if_fail (frame);
-	gtk_widget_set_sensitive (frame, TRUE);
-	spl->splpref.checkrules = TRUE;
-	spl->splrules.match_operator = ITDB_SPLMATCH_OR;
-    }
-}
-
-static void spl_none_radio_toggled (GtkToggleButton *togglebutton,
-				    GtkWidget *spl_window)
-{
-    Playlist *spl;
-    GtkWidget *frame;
-
-    g_return_if_fail (spl_window);
-    spl =  g_object_get_data (G_OBJECT (spl_window), "spl_work");
-    g_return_if_fail (spl);
-    frame = gtkpod_xml_get_widget (spl_window_xml, "spl_rules_frame");
-    g_return_if_fail (frame);
-    if (gtk_toggle_button_get_active (togglebutton))
-    {
-	gtk_widget_set_sensitive (frame, FALSE);
-	spl->splpref.checkrules = FALSE;
-    }
-}
-
 static void spl_matchcheckedonly_toggled (GtkToggleButton *togglebutton,
 					  GtkWidget *spl_window)
 {
@@ -635,10 +586,17 @@ static void splr_entry_changed (GtkEditable *editable,
     switch (type)
     {
     case spl_ET_FROMVALUE:
-	splr->fromvalue = atol (str);
-	if (splr->field == ITDB_SPLFIELD_RATING)
+	switch (splr->field)
 	{
-	    splr->fromvalue *= ITDB_RATING_STEP;
+	case ITDB_SPLFIELD_RATING:
+	    splr->fromvalue = ITDB_RATING_STEP * atol (str);
+	    break;
+	case ITDB_SPLFIELD_TIME:
+	    splr->fromvalue = 1000 * strtod (str, NULL);
+	    break;
+	default:
+	    splr->fromvalue = atol (str);
+	    break;
 	}
 	break;
     case spl_ET_FROMVALUE_DATE:
@@ -650,10 +608,17 @@ static void splr_entry_changed (GtkEditable *editable,
 	splr->fromdate = atol (str);
 	break;
     case spl_ET_TOVALUE:
-	splr->tovalue = atol (str);
-	if (splr->field == ITDB_SPLFIELD_RATING)
+	switch (splr->field)
 	{
-	    splr->tovalue *= ITDB_RATING_STEP;
+	case ITDB_SPLFIELD_RATING:
+	    splr->tovalue = ITDB_RATING_STEP * atol (str);
+	    break;
+	case ITDB_SPLFIELD_TIME:
+	    splr->tovalue = 1000 * strtod (str, NULL);
+	    break;
+	default:
+	    splr->tovalue = atol (str);
+	    break;
 	}
 	break;
     case spl_ET_TOVALUE_DATE:
@@ -1004,22 +969,27 @@ const gchar *entry_get_string (gchar *str, Itdb_SPLRule *splr,
 			       enum entrytype et)
 {
     gchar *strp = str;
-    gint stepsize = 1; /* for FROMVALUE/TOVALUE (20 for rating) */
 
     g_return_val_if_fail (str, NULL);
     g_return_val_if_fail (splr, NULL);
 
-    if (splr->field == ITDB_SPLFIELD_RATING)
-    {
-	stepsize = ITDB_RATING_STEP;
-    }
-
     switch (et)
     {
     case spl_ET_FROMVALUE:
 	if (splr->fromvalue == ITDB_SPL_DATE_IDENTIFIER)
 	    splr->fromvalue = 0;
-	snprintf (str, WNLEN, "%lld", (long long int)(splr->fromvalue / stepsize));
+	switch (splr->field)
+	{
+	case ITDB_SPLFIELD_RATING:
+	    snprintf (str, WNLEN, "%lld", (long long int)(splr->fromvalue / ITDB_RATING_STEP));
+	    break;
+	case ITDB_SPLFIELD_TIME:
+	    snprintf (str, WNLEN, "%.10g", ((gdouble)splr->fromvalue/1000));
+	    break;
+	default:
+	    snprintf (str, WNLEN, "%lld", (long long int)(splr->fromvalue));
+	    break;
+	}
 	break;
     case spl_ET_FROMVALUE_DATE:
 	if (splr->fromvalue == ITDB_SPL_DATE_IDENTIFIER)
@@ -1032,7 +1002,18 @@ const gchar *entry_get_string (gchar *str, Itdb_SPLRule *splr,
     case spl_ET_TOVALUE:
 	if (splr->tovalue == ITDB_SPL_DATE_IDENTIFIER)
 	    splr->tovalue = 0;
-	snprintf (str, WNLEN, "%lld",  (long long int)(splr->tovalue / stepsize));
+	switch (splr->field )
+	{
+	case ITDB_SPLFIELD_RATING:
+	    snprintf (str, WNLEN, "%lld",  (long long int)(splr->tovalue / ITDB_RATING_STEP));
+	    break;
+	case ITDB_SPLFIELD_TIME:
+	    snprintf (str, WNLEN, "%.10g", ((gdouble)splr->tovalue/1000));
+	    break;
+	default:
+	    snprintf (str, WNLEN, "%lld", (long long int)(splr->tovalue));
+	    break;
+	}
 	break;
     case spl_ET_TOVALUE_DATE:
 	if (splr->tovalue == ITDB_SPL_DATE_IDENTIFIER)
@@ -1578,7 +1559,7 @@ void spl_edit_all (iTunesDB *itdb, Playlist *spl, gint32 pos)
     g_return_if_fail (spl->is_spl);
     g_return_if_fail (itdb != NULL);
 
-    spl_window_xml = glade_xml_new (xml_file, "spl_window", NULL);
+    spl_window_xml = gtkpod_xml_new (xml_file, "spl_window");
     spl_window = gtkpod_xml_get_widget (spl_window_xml, "spl_window");
     
     g_return_if_fail (spl_window != NULL);
@@ -1591,69 +1572,50 @@ void spl_edit_all (iTunesDB *itdb, Playlist *spl, gint32 pos)
     g_object_set_data (G_OBJECT (spl_window), "spl_work", spl_dup);
     g_object_set_data (G_OBJECT (spl_window), "spl_pos",  GINT_TO_POINTER(pos));
     g_object_set_data (G_OBJECT (spl_window), "spl_itdb", itdb);
-    /* Set checkboxes and connect signal handlers */
+
+	/* Set checkboxes and connect signal handlers */
     if ((w = gtkpod_xml_get_widget (spl_window_xml, "spl_name_entry")))
     {
-	if (spl_dup->name)
-	    gtk_entry_set_text (GTK_ENTRY (w), spl_dup->name);	
-    }
-
-    if ((w = gtkpod_xml_get_widget (spl_window_xml, "spl_all_radio")))
-    {
-	g_signal_connect (w, "toggled",
-			  G_CALLBACK (spl_all_radio_toggled),
-			  spl_window);
-	gtk_toggle_button_set_active (
-	    GTK_TOGGLE_BUTTON (w),
-	    (spl_dup->splrules.match_operator == ITDB_SPLMATCH_AND));
-    }
-    if ((w = gtkpod_xml_get_widget (spl_window_xml, "spl_any_radio")))
-    {
-	g_signal_connect (w, "toggled",
-			  G_CALLBACK (spl_any_radio_toggled),
-			  spl_window);
-	gtk_toggle_button_set_active (
-	    GTK_TOGGLE_BUTTON (w),
-	    (spl_dup->splrules.match_operator == ITDB_SPLMATCH_OR));
-    }
-    if ((w = gtkpod_xml_get_widget (spl_window_xml, "spl_none_radio")))
-    {
-	g_signal_connect (w, "toggled",
-			  G_CALLBACK (spl_none_radio_toggled),
-			  spl_window);
-	gtk_toggle_button_set_active (
-	    GTK_TOGGLE_BUTTON (w), !spl_dup->splpref.checkrules);
+		if (spl_dup->name)
+			gtk_entry_set_text (GTK_ENTRY (w), spl_dup->name);	
     }
 
     if ((w = gtkpod_xml_get_widget (spl_window_xml, "spl_matchcheckedonly_button")))
     {
-	gtk_toggle_button_set_active (
-	    GTK_TOGGLE_BUTTON (w), spl_dup->splpref.matchcheckedonly);
-	g_signal_connect (w, "toggled",
-			  G_CALLBACK (spl_matchcheckedonly_toggled),
-			  spl_window);
+		gtk_toggle_button_set_active (
+			GTK_TOGGLE_BUTTON (w), spl_dup->splpref.matchcheckedonly);
+		g_signal_connect (w, "toggled",
+				  G_CALLBACK (spl_matchcheckedonly_toggled),
+				  spl_window);
     }
 
     if ((w = gtkpod_xml_get_widget (spl_window_xml, "spl_liveupdate_button")))
     {
-	gtk_toggle_button_set_active (
-	    GTK_TOGGLE_BUTTON (w), spl_dup->splpref.liveupdate);
-	g_signal_connect (w, "toggled",
-			  G_CALLBACK (spl_liveupdate_toggled),
-			  spl_window);
+		gtk_toggle_button_set_active (
+			GTK_TOGGLE_BUTTON (w), spl_dup->splpref.liveupdate);
+		g_signal_connect (w, "toggled",
+				  G_CALLBACK (spl_liveupdate_toggled),
+				  spl_window);
     }
 
     /* Signals for Cancel, OK, Delete */
     if ((w = gtkpod_xml_get_widget (spl_window_xml, "spl_cancel_button")))
     {
-	g_signal_connect (w, "clicked",
-		      G_CALLBACK (spl_cancel), spl_window);
+		g_signal_connect (w, "clicked",
+				  G_CALLBACK (spl_cancel), spl_window);
     }
-    if ((w = gtkpod_xml_get_widget (spl_window_xml, "spl_ok_button")))
+
+	if ((w = gtkpod_xml_get_widget (spl_window_xml, "spl_ok_button")))
     {
-	g_signal_connect (w, "clicked",
-		      G_CALLBACK (spl_ok), spl_window);
+		g_signal_connect (w, "clicked",
+				  G_CALLBACK (spl_ok), spl_window);
     }
+	
+	if ((w = gtkpod_xml_get_widget (spl_window_xml, "spl_match_rules")))
+    {
+		gtk_combo_box_set_active (GTK_COMBO_BOX (w), spl_MATCH_ALL);
+    }
+	
     g_signal_connect (spl_window, "delete_event",
 		      G_CALLBACK (spl_delete_event), spl_window);
 
@@ -1667,6 +1629,7 @@ void spl_edit_all (iTunesDB *itdb, Playlist *spl, gint32 pos)
     if ((defx != 0) && (defy != 0))
 	gtk_window_set_default_size (GTK_WINDOW (spl_window), defx, defy);
 
+	glade_xml_signal_autoconnect (spl_window_xml);
     gtk_widget_show (spl_window);
 
     block_widgets ();
@@ -1690,3 +1653,37 @@ void spl_edit_new (iTunesDB *itdb, gchar *name, gint32 pos)
 
     spl_edit_all (itdb, spl, pos);
 }
+
+/*
+	glade callback
+*/
+G_MODULE_EXPORT void spl_match_rules_changed (GtkComboBox *sender, gpointer e)
+{
+    Playlist *spl;
+	GtkWidget *spl_window = gtkpod_xml_get_widget (spl_window_xml, "spl_window");
+	GtkWidget *frame = gtkpod_xml_get_widget (spl_window_xml, "spl_rules_frame");
+
+    g_return_if_fail (spl_window);
+	g_return_if_fail (frame);
+    spl =  g_object_get_data (G_OBJECT (spl_window), "spl_work");
+    g_return_if_fail (spl);
+	
+	switch (gtk_combo_box_get_active (sender))
+	{
+	case spl_MATCH_ANY:
+		gtk_widget_set_sensitive (frame, TRUE);
+		spl->splpref.checkrules = TRUE;
+		spl->splrules.match_operator = ITDB_SPLMATCH_OR;
+		break;
+	case spl_MATCH_ALL:
+		gtk_widget_set_sensitive (frame, TRUE);
+		spl->splpref.checkrules = TRUE;
+		spl->splrules.match_operator = ITDB_SPLMATCH_AND;
+		break;
+	default:
+		gtk_widget_set_sensitive (frame, FALSE);
+		spl->splpref.checkrules = FALSE;
+		break;
+    }
+}
+
