@@ -25,7 +25,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: file_convert.c 1577 2007-06-23 02:07:43Z jcsjcs $
+|  $Id: file_convert.c 2218 2009-01-17 10:54:46Z jcsjcs $
 */
 
 
@@ -38,6 +38,7 @@
 #include "info.h"
 #include "misc.h"
 #include "misc_track.h"
+#include "mp3file.h"
 #include "prefs.h"
 #include <errno.h>
 #include <glib/gstdio.h>
@@ -160,6 +161,16 @@ static void transfer_activate (Conversion *conv, iTunesDB *itdb, gboolean active
 static void transfer_reset (Conversion *conv, iTunesDB *itdb);
 static void transfer_reschedule (Conversion *conv, iTunesDB *itdb);
 
+struct GaplessData
+{
+    guint32 pregap;       /* number of pregap samples */
+    guint64 samplecount;  /* number of actual music samples */
+    guint32 postgap;      /* number of postgap samples */
+    guint32 gapless_data; /* number of bytes from the first sync frame
+			   * to the 8th to last frame */
+    guint16 gapless_track_flag;
+};
+
 struct _Conversion
 {
     GMutex *mutex;          /* shared lock                              */
@@ -216,7 +227,7 @@ struct _ConvTrack
     gchar *fname_root;      /* filename root of converted file          */
     gchar *fname_extension; /* filename extension of converted file     */
     GPid  pid;              /* PID of child doing the conversion        */
-    gint  stderr;           /* stderr of child doing the conversion     */
+    gint  child_stderr;     /* stderr of child doing the conversion     */
     Track *track;           /* for reference, don't access inside threads! */
     iTunesDB *itdb;         /* for reference, don't access inside threads! */
     gint  threadnum;        /* number of thread working on this track   */
@@ -230,6 +241,7 @@ struct _ConvTrack
     gchar *genre;
     gchar *year;
     gchar *comment;
+    struct GaplessData gapless;   /* only used for MP3 */
     /* needed for transfering */
     gchar *dest_filename;
     gchar *mountpoint;
@@ -294,7 +306,7 @@ void file_convert_init ()
     conversion->dirsize = CONV_DIRSIZE_INVALID;
 
     /* setup log window */
-    log_xml = glade_xml_new (xml_file, "conversion_log", NULL);
+    log_xml = gtkpod_xml_new (xml_file, "conversion_log");
     conversion->log_window = gtkpod_xml_get_widget (log_xml, "conversion_log");
     gtk_window_set_default_size (GTK_WINDOW (conversion->log_window),
 				 prefs_get_int (FILE_CONVERT_LOG_SIZE_X),
@@ -490,7 +502,7 @@ static void conversion_display_hide_log_window (Conversion *conv)
     GtkWidget *mi;
     /* show display log if it was previously hidden and should be
        shown again */
-    mi = gtkpod_xml_get_widget (main_window_xml, "conversion_log");
+    mi = gtkpod_xml_get_widget (main_window_xml, "view_conversion_log");
     if (prefs_get_int (FILE_CONVERT_DISPLAY_LOG))
     {
 	if (conv->log_window_hidden && !conv->log_window_shown)
@@ -554,19 +566,28 @@ static void conversion_log_set_status (Conversion *conv)
     glthread = conv->threads;
     for (glpage=conv->pages; glpage; glpage=glpage->next)
     {
+	GtkNotebook *notebook = GTK_NOTEBOOK (conv->notebook);
 	GtkWidget *child = glpage->data;
+	const gchar *current_label;
 	g_return_if_fail (child);
 
+	current_label = gtk_notebook_get_tab_label_text (notebook, child);
+
 	/* in the beginning we may have more pages than thread entries */
 	if (glthread && glthread->data)
 	{
-	    gtk_notebook_set_tab_label_text (GTK_NOTEBOOK (conv->notebook),
-					     child, _("active"));
+	    if (!current_label || strcmp (current_label, _("active")) != 0)
+	    {   /* only change the label if it has changed --
+		   otherwise our tooltips will be switched off */
+		gtk_notebook_set_tab_label_text (notebook, child, _("active"));
+	    }
 	}
 	else
 	{
-	    gtk_notebook_set_tab_label_text (GTK_NOTEBOOK (conv->notebook),
-					     child, _("inactive"));
+	    if (!current_label || strcmp (current_label, _("inactive")) != 0)
+	    {
+		gtk_notebook_set_tab_label_text (notebook, child, _("inactive"));
+	    }
 	}
 
 	if (glthread)
@@ -739,7 +760,7 @@ static void conversion_log_append (Conversion *conv,
     GtkWidget *textview;
     GtkTextBuffer *textbuffer;
     GtkTextIter start, end;
-    const gchar *ptr, *next;
+    const gchar *run, *ptr, *next;
 
     g_return_if_fail (conv);
 
@@ -753,13 +774,18 @@ static void conversion_log_append (Conversion *conv,
     textbuffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview));
     gtk_text_buffer_get_end_iter (textbuffer, &end);
 
-    ptr = text;
+    run = ptr = text;
 
     while (*ptr)
     {
 	next = g_utf8_find_next_char (ptr, NULL);
 	if (*ptr == '\b')
 	{
+	    if (ptr > run)
+	    {
+		gtk_text_buffer_insert (textbuffer, &end, run, ptr - run);
+	    }
+	    run = next;
 	    start = end;
 	    if (gtk_text_iter_backward_char (&start))
 	    {
@@ -768,16 +794,21 @@ static void conversion_log_append (Conversion *conv,
 	}
 	else if(*ptr == '\r')
 	{
+	    if (ptr > run)
+	    {
+		gtk_text_buffer_insert (textbuffer, &end, run, ptr - run);
+	    }
+	    run = next;
 	    start = end;
 	    gtk_text_iter_set_line_offset (&start, 0);
 	    gtk_text_buffer_delete (textbuffer, &start, &end);
 	}
-	else
-	{
-	    gtk_text_buffer_insert (textbuffer, &end, ptr, next - ptr);
-	}
 	ptr = next;
     }
+    if (ptr > run)
+    {
+	gtk_text_buffer_insert (textbuffer, &end, run, ptr - run);
+    }
     gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW (textview),
 				  &end, 0.0, FALSE, 0.0, 0.0); 
 
@@ -1024,19 +1055,26 @@ static gboolean conversion_add_track (Conversion *conv, Track *track)
     ConvTrack *ctr;
     gchar *conversion_cmd = NULL;
     const gchar *typestr = NULL;
-    gboolean convert=FALSE, must_convert = FALSE;
+    gboolean convert = FALSE, must_convert = FALSE;
     gboolean result = TRUE;
 
+    debug ("entering conversion_add_track\n");
+
     g_return_val_if_fail (conv, FALSE);
     g_return_val_if_fail (track, FALSE);
     g_return_val_if_fail (track->itdb, FALSE);
     etr = track->userdata;
     g_return_val_if_fail (etr, FALSE);
 
-    if ((track->itdb->usertype & GP_ITDB_TYPE_LOCAL) ||
-	(track->transferred))
-    {   /* no conversion or transfer needed */
-	return TRUE;
+    debug ("considering '%s' to conversion/transfer list\n", etr->pc_path_locale);
+
+    if ((track->itdb->usertype & GP_ITDB_TYPE_LOCAL) || (track->transferred))
+    {
+      debug ("adding aborted: lcl:%d, trsfrd:%d\n",
+	     track->itdb->usertype & GP_ITDB_TYPE_LOCAL,
+	     track->transferred);
+		/* no conversion or transfer needed */
+		return TRUE;
     }
 
     /* Create ConvTrack structure */
@@ -1057,33 +1095,33 @@ static gboolean conversion_add_track (Conversion *conv, Track *track)
 
     if (!etr->pc_path_locale || (strlen (etr->pc_path_locale) == 0))
     {
-	gchar *buf = get_track_info (track, FALSE);
-	gtkpod_warning (_("Original filename not available for '%s.'\n"), buf);
-	g_free (buf);
+		gchar *buf = get_track_info (track, FALSE);
+		gtkpod_warning (_("Original filename not available for '%s.'\n"), buf);
+		g_free (buf);
 
-	etr->conversion_status = FILE_CONVERT_FAILED;
-	/* add to failed list */
-	g_mutex_lock (conv->mutex);
-	conv->failed = g_list_prepend (conv->failed, ctr);
-	g_mutex_unlock (conv->mutex);
-	debug ("added track to failed %p\n", track);
-	return FALSE;
+		etr->conversion_status = FILE_CONVERT_FAILED;
+		/* add to failed list */
+		g_mutex_lock (conv->mutex);
+		conv->failed = g_list_prepend (conv->failed, ctr);
+		g_mutex_unlock (conv->mutex);
+		debug ("added track to failed %p\n", track);
+		return FALSE;
     }
 
     if (!g_file_test (etr->pc_path_locale, G_FILE_TEST_IS_REGULAR))
     {
-	gchar *buf = get_track_info (track, FALSE);
-	gtkpod_warning (_("Filename '%s' is no longer valid for '%s'.\n"),
-			etr->pc_path_utf8, buf);
-	g_free (buf);
+		gchar *buf = get_track_info (track, FALSE);
+		gtkpod_warning (_("Filename '%s' is no longer valid for '%s'.\n"),
+				etr->pc_path_utf8, buf);
+		g_free (buf);
 
-	etr->conversion_status = FILE_CONVERT_FAILED;
-	/* add to failed list */
-	g_mutex_lock (conv->mutex);
-	conv->failed = g_list_prepend (conv->failed, ctr);
-	g_mutex_unlock (conv->mutex);
-	debug ("added track to failed %p\n", track);
-	return FALSE;
+		etr->conversion_status = FILE_CONVERT_FAILED;
+		/* add to failed list */
+		g_mutex_lock (conv->mutex);
+		conv->failed = g_list_prepend (conv->failed, ctr);
+		g_mutex_unlock (conv->mutex);
+		debug ("added track to failed %p\n", track);
+		return FALSE;
     }	
 
     /* Find the correct script for conversion */
@@ -1109,28 +1147,28 @@ static gboolean conversion_add_track (Conversion *conv, Track *track)
 	    debug ("added track to finished %p\n", track);
             return TRUE;
         case FILE_TYPE_M4A:
-	    convert = prefs_get_int ("convert_m4a");
             conversion_cmd = prefs_get_string ("path_conv_m4a");
+			convert = conversion_cmd && conversion_cmd[0] && prefs_get_int ("convert_m4a");
             break;
         case FILE_TYPE_WAV:
-	    convert = prefs_get_int ("convert_wav");
             conversion_cmd = prefs_get_string ("path_conv_wav");
+			convert = conversion_cmd && conversion_cmd[0] && prefs_get_int ("convert_wav");
             break;
         case FILE_TYPE_MP3:
-	    convert = prefs_get_int ("convert_mp3");
             conversion_cmd = prefs_get_string ("path_conv_mp3");
+			convert = conversion_cmd && conversion_cmd[0] && prefs_get_int ("convert_mp3");
             break;
         case FILE_TYPE_OGG:
-	    convert = prefs_get_int ("convert_ogg");
-	    conversion_cmd = prefs_get_string ("path_conv_ogg");
-	    must_convert = TRUE;
-	    typestr = _("Ogg Vorbis");
+			conversion_cmd = prefs_get_string ("path_conv_ogg");
+			convert = conversion_cmd && conversion_cmd[0] && prefs_get_int ("convert_ogg");
+			must_convert = TRUE;
+			typestr = _("Ogg Vorbis");
             break;
         case FILE_TYPE_FLAC:
-	    convert = prefs_get_int ("convert_flac");
-	    conversion_cmd = prefs_get_string ("path_conv_flac");
-	    must_convert = TRUE;
-	    typestr = _("FLAC");
+			conversion_cmd = prefs_get_string ("path_conv_flac");
+			convert = conversion_cmd && conversion_cmd[0] && prefs_get_int ("convert_flac");
+			must_convert = TRUE;
+			typestr = _("FLAC");
             break;
     }
 
@@ -1482,9 +1520,11 @@ static gboolean conversion_scheduler_unlocked (Conversion *conv)
 
 	    if (ctr->valid)
 	    {
-		GList *trackgl;
+		GList *trackgl, *tracks;
+
 		g_return_val_if_fail (ctr->track, TRUE);
-		GList *tracks = gp_itdb_find_same_tracks_in_itdbs (ctr->track);
+
+		tracks = gp_itdb_find_same_tracks_in_itdbs (ctr->track);
 		for (trackgl=tracks; trackgl; trackgl=trackgl->next)
 		{
 		    ExtraTrackData *etr;
@@ -1511,6 +1551,11 @@ static gboolean conversion_scheduler_unlocked (Conversion *conv)
 			etr->converted_file = g_strdup (ctr->converted_file);
 			etr->conversion_status = FILE_CONVERT_CONVERTED;
 			tr->size = ctr->converted_size;
+			tr->pregap = ctr->gapless.pregap;
+			tr->samplecount = ctr->gapless.samplecount;
+			tr->postgap = ctr->gapless.postgap;
+			tr->gapless_data = ctr->gapless.gapless_data;
+			tr->gapless_track_flag = ctr->gapless.gapless_track_flag;
 			pm_track_changed (tr);
 			data_changed (tr->itdb);
 		    }
@@ -1665,6 +1710,8 @@ static gboolean conversion_scheduler_unlocked (Conversion *conv)
 			g_free (ctr->errormessage);
 			ctr->errormessage = NULL;
 		    }
+		    g_free (ctr->dest_filename);
+		    ctr->dest_filename = NULL;
 		    tri->finished = g_list_prepend (tri->finished, ctr);
 		}
 		else
@@ -2076,7 +2123,7 @@ static gchar *conversion_get_fname_extension (Conversion *conv, ConvTrack *ctr)
 	{
 	    gchar *buf = conversion_get_track_info (NULL, ctr);
 	    ctr->errormessage =
-		g_strdup_printf (_("Conversion of '%s' failed: '%s %s' did not return filename extension as expected.\n\n"),
+		g_strdup_printf (_("Conversion of '%s' failed: '\"%s\" %s' did not return filename extension as expected.\n\n"),
 				 buf,
 				 argv[0], argv[1]);
 	    g_free (buf);
@@ -2290,13 +2337,14 @@ static gboolean conversion_convert_track (Conversion *conv, ConvTrack *ctr)
 				  argv,         /* command line   */
 				  NULL,         /* envp           */
 				  G_SPAWN_SEARCH_PATH |
-				  G_SPAWN_DO_NOT_REAP_CHILD,
+				  G_SPAWN_DO_NOT_REAP_CHILD |
+				  G_SPAWN_STDOUT_TO_DEV_NULL,
 				  pgid_setup,   /* setup func     */
 				  NULL,         /* user data      */
 				  &ctr->pid,    /* child's PID    */
 				  NULL,         /* child's stdin  */
 				  NULL,         /* child's stdout */
-				  &ctr->stderr, /* child's stderr */
+				  &ctr->child_stderr, /* child's stderr */
 				  &error);
 
 	    child_pid = ctr->pid;
@@ -2322,7 +2370,7 @@ static gboolean conversion_convert_track (Conversion *conv, ConvTrack *ctr)
 		gint status;
 
 		/* set up i/o channel to main thread */
-		ctr->gio_channel = g_io_channel_unix_new (ctr->stderr);
+		ctr->gio_channel = g_io_channel_unix_new (ctr->child_stderr);
 		g_io_channel_set_flags (ctr->gio_channel,
 					G_IO_FLAG_NONBLOCK, NULL);
 		g_io_channel_set_close_on_unref (ctr->gio_channel, TRUE);
@@ -2390,6 +2438,51 @@ static gboolean conversion_convert_track (Conversion *conv, ConvTrack *ctr)
 	}
     }
 
+    /* Fill in additional info (currently only gapless info for MP3s */
+    if (result == TRUE)
+    {
+	Track *track;
+	gboolean retval;
+
+	switch (determine_file_type (ctr->converted_file))
+	{
+        case FILE_TYPE_UNKNOWN:
+        case FILE_TYPE_M4P:
+        case FILE_TYPE_M4B:
+        case FILE_TYPE_M4V:
+        case FILE_TYPE_MP4:
+        case FILE_TYPE_MOV:
+        case FILE_TYPE_MPG:
+        case FILE_TYPE_M3U:
+        case FILE_TYPE_PLS:
+        case FILE_TYPE_IMAGE:
+        case FILE_TYPE_DIRECTORY:
+        case FILE_TYPE_M4A:
+        case FILE_TYPE_WAV:
+        case FILE_TYPE_OGG:
+        case FILE_TYPE_FLAC:
+            break;
+        case FILE_TYPE_MP3:
+	    g_mutex_unlock (conv->mutex);
+
+	    track = gp_track_new ();
+	    retval = mp3_read_gapless (ctr->converted_file, track);
+
+	    g_mutex_lock (conv->mutex);
+
+	    if (ctr->valid && (retval == TRUE))
+	    {
+		ctr->gapless.pregap = track->pregap;
+		ctr->gapless.samplecount = track->samplecount;
+		ctr->gapless.postgap = track->postgap;
+		ctr->gapless.gapless_data = track->gapless_data;
+		ctr->gapless.gapless_track_flag = track->gapless_track_flag;
+	    }
+	    itdb_track_free (track);
+            break;
+	}
+    }
+
     g_mutex_unlock (conv->mutex);
 
     return result;
@@ -2689,11 +2782,8 @@ static GList *transfer_get_failed_tracks (Conversion *conv, iTunesDB *itdb)
     tri = transfer_get_tri (conv, itdb);
     g_return_val_if_fail (tri, (g_mutex_unlock (conv->mutex), NULL));
 
-    if (conv->failed || tri->failed)
-    {   /* move the tracks over to tri->finished by calling the
-	   scheduler directly */
-	conversion_scheduler_unlocked (conv);
-    }
+    /* Make sure all failed tracks are forwarded to tri->finished */
+    conversion_scheduler_unlocked (conv);
 
     for (gl=tri->finished; gl; gl=gl->next)
     {
@@ -3018,8 +3108,6 @@ static FileTransferStatus transfer_transfer_track (TransferItdb *tri,
 	return result;
     }
 
-    g_mutex_unlock (conv->mutex);
-
     copy_success = itdb_cp (source_file, dest_file, &error);
 
     g_mutex_lock (conv->mutex);
