@@ -24,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: misc.c 1558 2007-06-18 15:33:07Z jcsjcs $
+|  $Id: misc.c 2079 2008-08-02 10:50:09Z jcsjcs $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -45,7 +45,8 @@
 #include "misc.h"
 #include "prefs.h"
 #include "misc_track.h"
-
+#include "display_photo.h"
+#include "stock_icons.h"
 
 #define DEBUG_MISC 0
 
@@ -54,222 +55,66 @@
 const gchar *SCRIPTDIR = PACKAGE_DATA_DIR G_DIR_SEPARATOR_S PACKAGE G_DIR_SEPARATOR_S "scripts" G_DIR_SEPARATOR_S;
 
 
-/*------------------------------------------------------------------*\
- *                                                                  *
- *             About Window                                         *
- *                                                                  *
-\*------------------------------------------------------------------*/
-
-static GtkWidget *about_window = NULL;
-
-/* ATTENTION: directly used as callback in gtkpod.glade -- if you
-   change the arguments of this function make sure you define a
-   separate callback for gtkpod.glade */
-void open_about_window ()
-{
-  GtkLabel *about_label;
-  gchar *label_text;
-  GtkTextView *textview;
-  GtkTextIter ti;
-  GtkTextBuffer *tb;
-  GladeXML *about_xml;
-
-  if (about_window != NULL) return;
-  /* about_window = create_gtkpod_about_window (); */
-
-  about_xml = glade_xml_new (xml_file, "gtkpod_about_window", NULL);
-  glade_xml_signal_autoconnect (about_xml);
-  about_window = gtkpod_xml_get_widget (about_xml, "gtkpod_about_window");
-  
-  
-  about_label = GTK_LABEL (gtkpod_xml_get_widget (about_xml, "about_label"));
-  label_text = g_strdup_printf (_("gtkpod Version %s: Cross-Platform Multi-Lingual Interface to Apple's iPod(tm)."), VERSION);
-  gtk_label_set_text (about_label, label_text);
-  g_free (label_text);
-  {
-      gchar *text[] = {_("\
-(C) 2002 - 2007\n\
-Jorg Schuler (jcsjcs at users dot sourceforge dot net)\n\
-Corey Donohoe (atmos at atmos dot org)\n\
-\n\
-\n"),
-		       _("\
-This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.\n\
-\n\
-This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.\n\
-\n\
-You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.\n\
-\n\
-\n"),
-		       _("\
-Patches were supplied by the following people (list may be incomplete -- please contact me)\n\n"),
-		       _("\
-Ramesh Dharan: Multi-Edit (edit tags of several tracks in one run)\n"),
-		       _("\
-Hiroshi Kawashima: Japanese charset autodetecion feature\n"),
-		       _("\
-Adrian Ulrich: porting of playlist code from mktunes.pl to itunesdb.c\n"),
-		       _("\
-Walter Bell: correct handling of DND URIs with escaped characters and/or cr/newlines at the end\n"),
-		       _("\
-Sam Clegg: user defined filenames when exporting tracks from the iPod\n"),
-		       _("\
-Chris Cutler: automatic creation of various playlist types\n"),
-		       _("\
-Graeme Wilford: reading and writing of the 'Composer' ID3 tags, progress dialogue during sync\n"),
-		       _("\
-Edward Matteucci: debugging, special playlist creation, most of the volume normalizing code\n"),
-		       _("\
-Jens Lautenbach: some optical improvements\n"),
-		       _("\
-Alex Tribble: iPod eject patch\n"),
-		       _("\
-Yaroslav Halchenko: Orphaned and dangling tracks handling\n"),
-		       _("\
-Andrew Huntwork: Filename case sensitivity fix and various other bugfixes\n"),
-		       _("\
-Ero Carrera: Filename validation and quick sync when copying tracks from the iPod\n"),
-		       _("\
-Jens Taprogge: Support for LAME's replay gain tag to normalize volume\n"),
-		       _("\
-Armando Atienza: Support with external playcounts\n"),
-		       _("\
-D.L. Sharp: Support for m4b files (bookmarkable AAC files)\n"),
-		       _("\
-Jim Hall: Decent INSTALL file\n"),
-		       _("\
-Juergen Helmers, Markus Gaugusch: Conversion scripts to sync calendar/contacts to the iPod\n"),    /* J"urgen! */
-		       _("\
-Flavio Stanchina: bugfixes\n"),
-		       _("\
-Chris Micacchi: when sorting ignore 'the' and similar at the beginning of the title\n"),
-		       _("\
-Steve Jay: use statvfs() instead of df (better portability, faster)\n"),
-		       "\n",
-		       _("\
-Christoph Kunz: address compatibility issues when writing id3v2.4 type mp3 tags\n"),
-		       "\n",
-		       _("\
-James Liggett:\n\
-     replacement of old GTK file selection dialogs with new GTK filechooser dialogs\n\
-     refactored user preferences system.\n"),
-                       "\n",
-		       _("\
-Daniel Kercher: sync scripts for abook and webcalendar\n"),
-		       "\n",
-		       _("\
-Clinton Gormley: sync scripts for thunderbird\n"),
-		       "\n",
-		       _("\
-Sebastien Beridot: sync script for ldif addressbook format\n"),
-		       "\n",
-		       _("\
-Sebastian Scherer: sync script for kNotes\n"),
-		       "\n",
-		       _("\
-Nick Piper: sync script for Palm, type-ahead search\n"),
-		       "\n",
-		       _("\
-Uwe Hermann: help with support for iPod Video\n"),
-		       "\n",
-		       _("\
-Iain Benson: support for compilation tag in mp3 files and separate display of compilations in the sort tab.\n"),
-		       _("\
-Nicolas Chariot: icons of buttons\n\
-\n\
-\n"),
-		       _("\
-This program borrows code from the following projects:\n"),
-		       _("\
-    gnutools: (mktunes.pl, ported to C) reading and writing of iTunesDB (http://www.gnu.org/software/gnupod/)\n"),
-		       _("\
-    iPod.cpp, iPod.h by Samuel Wood (sam dot wood at gmail dot com): some code for smart playlists is based on his C++-classes.\n"),
-		       _("\
-    mp3info:  mp3 playlength detection (http://ibiblio.org/mp3info/)\n"),
-		       _("\
-    xmms:     dirbrowser, mp3 playlength detection (http://www.xmms.org)\n"),
-		       "\n",
-		       _("\
-The GUI was created with the help of glade-2 (http://glade.gnome.org/).\n"),
-		       NULL };
-      gchar **strp = text;
-      textview = GTK_TEXT_VIEW (gtkpod_xml_get_widget (about_xml, "credits_textview"));
-      tb = gtk_text_view_get_buffer (textview);
-      while (*strp)
-      {
-	  gtk_text_buffer_get_end_iter (tb, &ti);
-	  gtk_text_buffer_insert (tb, &ti, *strp, -1);
-	  ++strp;
-      }
-  }
-
- {
-     gchar  *text[] = { _("\
-French:   David Le Brun (david at dyn-ns dot net)\n"),
-				     _("\
-German:   Jorg Schuler (jcsjcs at users dot sourceforge dot net)\n"),
-			             _("\
-Hebrew: Assaf Gillat (gillata at gmail dot com)\n"),
-				     _("\
-Italian:  Edward Matteucci (edward_matteucc at users dot sourceforge dot net)\n"),
-				     _("\
-Japanese: Ayako Sano\n"),
-				     _("\
-Japanese: Kentaro Fukuchi (fukuchi at users dot sourceforge dot net)\n"),
-				     _("\
-Swedish: Stefan Asserhall (stefan asserhall at comhem dot se)\n"),
-				     NULL };
-      gchar **strp = text;
-      textview = GTK_TEXT_VIEW (gtkpod_xml_get_widget (about_xml, "translators_textview"));
-      tb = gtk_text_view_get_buffer (textview);
-      while (*strp)
-      {
-	  gtk_text_buffer_get_end_iter (tb, &ti);
-	  gtk_text_buffer_insert (tb, &ti, *strp, -1);
-	  ++strp;
-      }
-  }
-
-  gtk_widget_show (about_window);
-}
-
-
-/* callback for close button */
-void
-on_about_window_close_button           (GtkButton       *button,
-					gpointer         user_data)
-{
-  close_about_window (); /* in misc.c */
-}
-
-
-/* callback for window close button */
-gboolean
-on_about_window_close                  (GtkWidget       *widget,
-					GdkEvent        *event,
-					gpointer         user_data)
-{
-  close_about_window (); /* in misc.c */
-  return FALSE;
-}
-
-
-void close_about_window (void)
-{
-  g_return_if_fail (about_window != NULL);
-  gtk_widget_destroy (about_window);
-  about_window = NULL;
-}
-
-
-
-
 /*------------------------------------------------------------------*\
  *                                                                  *
  *             Miscellaneous                                        *
  *                                                                  *
 \*------------------------------------------------------------------*/
 
+/* Copied g_utf8_strcasestr from GtkSourceView */
+gchar *utf8_strcasestr (const gchar *haystack, const gchar *needle)
+{
+	gsize needle_len;
+	gsize haystack_len;
+	gchar *ret = NULL;
+	gchar *p;
+	gchar *casefold;
+	gchar *caseless_haystack;
+	gint i;
+
+	g_return_val_if_fail (haystack != NULL, NULL);
+	g_return_val_if_fail (needle != NULL, NULL);
+
+	casefold = g_utf8_casefold (haystack, -1);
+	caseless_haystack = g_utf8_normalize (casefold, -1, G_NORMALIZE_ALL);
+	g_free (casefold);
+
+	needle_len = g_utf8_strlen (needle, -1);
+	haystack_len = g_utf8_strlen (caseless_haystack, -1);
+
+	if (needle_len == 0)
+	{
+		ret = (gchar *)haystack;
+		goto finally_1;
+	}
+
+	if (haystack_len < needle_len)
+	{
+		ret = NULL;
+		goto finally_1;
+	}
+
+	p = (gchar*)caseless_haystack;
+	needle_len = strlen (needle);
+	i = 0;
+
+	while (*p)
+	{
+		if ((strncmp (p, needle, needle_len) == 0))
+		{
+			ret = g_utf8_offset_to_pointer (haystack, i);
+			goto finally_1;
+		}
+
+		p = g_utf8_next_char (p);
+		i++;
+	}
+
+finally_1:
+	g_free (caseless_haystack);
+
+	return ret;
+}
 
 /* Calculate the time in ms passed since @old_time. @old_time is
    updated with the current time if @update is TRUE*/
@@ -324,7 +169,34 @@ parse_tracks_from_string(gchar **s, Track **track)
     return FALSE;
 }
 
+gboolean
+parse_artwork_from_string(gchar **s, Artwork **artwork)
+{
+    g_return_val_if_fail (artwork, FALSE);
+    *artwork = NULL;
+    g_return_val_if_fail (s, FALSE);
 
+    if(*s)
+    {
+	gchar *str = *s;
+	gchar *strp = strchr (str, '\n');
+	int tokens;
+
+	if (strp == NULL)
+	{
+	    *artwork = NULL;
+	    *s = NULL;
+	    return FALSE;
+	}
+	tokens = sscanf (str, "%p", artwork);
+	++strp;
+	if (*strp) *s = strp;
+	else       *s = NULL;
+	if (tokens == 1) 	return TRUE;
+	else                    return FALSE;
+    }
+    return FALSE;
+}
 
 /* Duplicate a GList (shallow copy) */
 GList *glist_duplicate (GList *list)
@@ -427,6 +299,11 @@ void call_script (gchar *script, ...)
 
 /* Create a NULL-terminated array of strings given in the command
    line. The last argument must be NULL.
+
+   As a special feature, the first argument is split up into
+   individual strings to allow the use of "convert-2mp3 -q <special
+   settings>". Set the first argument to NULL if you don't want this.
+
    You must free the returned array with g_strfreev() after use. */
 gchar **build_argv_from_strings (const gchar *first_arg, ...)
 {
@@ -435,19 +312,31 @@ gchar **build_argv_from_strings (const gchar *first_arg, ...)
     const gchar *str;
     GPtrArray *ptra = g_ptr_array_sized_new (20);
 
-    g_ptr_array_add (ptra, g_strdup (first_arg));
-    /* add the rest of the strings */
     if (first_arg)
     {
-	va_start (args, first_arg);
-	do
+	gchar **strings = g_strsplit (first_arg, " ", 0);
+	gchar **strp = strings;
+	while (*strp)
 	{
-	    str = va_arg (args, const gchar *);
-	    g_ptr_array_add (ptra, g_strdup (str));
+	    if (**strp)
+	    {   /* ignore empty strings */
+		g_ptr_array_add (ptra, g_strdup(*strp));
+	    }
+	    ++strp;
 	}
-	while (str);
-	va_end (args);
+	g_strfreev (strings);
     }
+
+    va_start (args, first_arg);
+    do
+    {
+	str = va_arg (args, const gchar *);
+	g_ptr_array_add (ptra, g_strdup (str));
+    }
+    while (str);
+
+    va_end (args);
+
     argv = (gchar **)g_ptr_array_free (ptra, FALSE);
 
     return argv;
@@ -1388,6 +1277,28 @@ gint64 get_size_of_directory (const gchar *dir)
 
 
 
+/**
+ * Wrapper for glade_xml_new() for cygwin compatibility issues
+ *
+ **/
+GladeXML *gtkpod_xml_new (const gchar *xml_file, const gchar *name)
+{
+    GladeXML *xml;
+	
+#ifdef ENABLE_NLS
+	xml = glade_xml_new (xml_file, name, GETTEXT_PACKAGE);
+#else
+	xml = glade_xml_new (xml_file, name, NULL);
+#endif
+
+    if (!xml)
+		fprintf (stderr, "*** Programming error: Cannot create glade XML: '%s'\n",
+				 name);
+
+    return xml;
+}
+
+
 /**
  * Wrapper for gtkpod_xml_get_widget() giving out a warning if widget
  * could not be found.
@@ -1395,16 +1306,15 @@ gint64 get_size_of_directory (const gchar *dir)
  **/
 GtkWidget *gtkpod_xml_get_widget (GladeXML *xml, const gchar *name)
 {
-    GtkWidget *w=glade_xml_get_widget (xml, name);
+    GtkWidget *w = glade_xml_get_widget (xml, name);
 
     if (!w)
-	fprintf (stderr, "*** Programming error: Widget not found: '%s'\n",
-		 name);
+		fprintf (stderr, "*** Programming error: Widget not found: '%s'\n",
+				 name);
 
     return w;
 }
 
-
 /* ------------------------------------------------------------
  *
  *        Helper functions for pref keys
@@ -1780,12 +1690,14 @@ static void load_ipod_index_prefs (gint index, const gchar *mountpoint)
 	GError *error = NULL;
 	gchar *path = g_build_filename (dir, "gtkpod.prefs", NULL);
 	temp_prefs = temp_prefs_load (path, &error);
+	g_free (path);
 	if (temp_prefs)
 	{
 	    gchar *subkey;
 	    subkey = get_itdb_prefs_key (index, "");
 	    /* rename 'itdb_*' to 'itdb_<index>_*' */
 	    temp_prefs_rename_subkey (temp_prefs, "itdb_", subkey);
+	    g_free (subkey);
 	    /* merge with real prefs */
 	    temp_prefs_apply (temp_prefs);
 	    /* destroy temp prefs */
@@ -1891,7 +1803,7 @@ void gtkpod_init (int argc, char *argv[])
     /* Initialisation of libxml */
     LIBXML_TEST_VERSION;
 
-    main_window_xml = glade_xml_new (xml_file, "gtkpod", NULL);
+    main_window_xml = gtkpod_xml_new (xml_file, "gtkpod");
 
     glade_xml_signal_autoconnect (main_window_xml);
   
@@ -1900,6 +1812,7 @@ void gtkpod_init (int argc, char *argv[])
     prefs_init (argc, argv); 
 
     coverart_init (argv[0]);
+    stockid_init (argv[0]);
 
     file_convert_init ();
 
@@ -1930,6 +1843,11 @@ void gtkpod_shutdown ()
     /* stop accepting requests for playcount updates */
     server_shutdown ();
 
+		/* Change the windows back to track view to ensure the
+		 * sorttab state is saved correctly/
+		 */
+		gphoto_change_to_photo_window (FALSE);
+		
     /* Sort column order needs to be stored */
     tm_store_col_order();
   
