@@ -1,4 +1,4 @@
-/* Time-stamp: <2007-12-11 21:31:57 jcs>
+/* Time-stamp: <2009-01-11 17:52:48 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: mp3file.c 1813 2007-12-11 12:34:20Z jcsjcs $
+|  $Id: mp3file.c 2204 2009-01-11 10:13:15Z jcsjcs $
 */
 
 
@@ -88,6 +88,7 @@ struct _File_Tag
     gchar *lyrics;        /* does not appear to be the full lyrics --
 			     only used to set the flag 'lyrics_flag'
 			     of the Track structure */
+    gchar *albumartist;         /* Album artist  */
 };
 
 
@@ -220,7 +221,7 @@ typedef struct {
 } MP3Header;
 
 typedef struct {
-    gchar *filename;
+    const gchar *filename;
     FILE *file;
     off_t datasize;
     gint header_isvalid;
@@ -234,11 +235,11 @@ typedef struct {
 } MP3Info;
 
 /* This is for xmms code */
-static guint get_track_time(gchar *path);
+static guint get_track_time(const gchar *path);
 
 
 /* This is for soundcheck code */
-gboolean mp3_read_lame_tag (gchar *path, LameTag *lt);
+gboolean mp3_read_lame_tag (const gchar *path, LameTag *lt);
 
 /* ------------------------------------------------------------
 
@@ -992,7 +993,7 @@ static guint get_track_time_file(FILE * file)
 	return 0;
 }
 
-static guint get_track_time (gchar *path)
+static guint get_track_time (const gchar *path)
 {
     guint result = 0;
 
@@ -1259,8 +1260,9 @@ static void id3_set_string (struct id3_tag *tag,
     /* Use the specified text encoding */
     field = id3_frame_field (frame, 0);
     id3_field_settextencoding(field, encoding);
-   
-    if (strcmp (frame_name, ID3_FRAME_COMMENT) == 0)
+
+    if ((strcmp (frame_name, ID3_FRAME_COMMENT) == 0) ||
+        (strcmp (frame_name, "USLT") == 0))
     {
 	field = id3_frame_field (frame, 3);
 	field->type = ID3_FIELD_TYPE_STRINGFULL;
@@ -1275,6 +1277,9 @@ static void id3_set_string (struct id3_tag *tag,
     /* maybe could be optimized see
        http://www.mars.org/mailman/public/mad-dev/2002-October/000739.html
     */
+/* don't handle the genre frame differently any more */
+#define USE_GENRE_IDS 0
+#if USE_GENRE_IDS
     if (strcmp (frame_name, ID3_FRAME_GENRE) == 0)
     {
 	id3_ucs4_t *tmp_ucs4 = id3_utf8_ucs4duplicate ((id3_utf8_t *)data);
@@ -1282,7 +1287,7 @@ static void id3_set_string (struct id3_tag *tag,
 	if (index != -1)
 	{
 	    /* valid genre -- simply store the genre number */
-	    gchar *tmp = g_strdup_printf("%d", index);
+	    gchar *tmp = g_strdup_printf("(%d)", index);
 	    ucs4 = id3_latin1_ucs4duplicate (tmp);
 	    g_free (tmp);
 	}
@@ -1309,6 +1314,7 @@ static void id3_set_string (struct id3_tag *tag,
     }
     else
     {
+#endif
 	if (encoding == ID3_FIELD_TEXTENCODING_ISO_8859_1)
 	{
 	    /* we read 'ISO_8859_1' to stand for 'any locale charset'
@@ -1323,9 +1329,11 @@ static void id3_set_string (struct id3_tag *tag,
 	       worry about charsets */
 	    ucs4 = id3_utf8_ucs4duplicate ((id3_utf8_t *)data);
 	}
+#if USE_GENRE_IDS
     }
+#endif
 
-    if (strcmp (frame_name, ID3_FRAME_COMMENT) == 0)
+    if ((strcmp (frame_name, ID3_FRAME_COMMENT) == 0) || (strcmp (frame_name, "USLT") == 0))
 	res = id3_field_setfullstring (field, ucs4);
     else
 	res = id3_field_setstrings (field, 1, &ucs4);
@@ -1429,6 +1437,97 @@ static gboolean id3_apic_read (const gchar *filename,
     return TRUE;
 }
 
+
+/* Do some checks on the genre string -- ideally this should
+ * be done within the id3tag library, I think */
+static void handle_genre_variations (gchar **genrep)
+{
+/* http://www.id3.org/id3v2.3.0#head-42b02d20fb8bf48e38ec5415e34909945dd849dc */
+
+/* The 'Content type', which previously was stored as a one byte
+ * numeric value only, is now a numeric string. You may use one or
+ * several of the types as ID3v1.1 did or, since the category list
+ * would be impossible to maintain with accurate and up to date
+ * categories, define your own.
+
+ * References to the ID3v1 genres can be made by, as first byte, enter
+ * "(" followed by a number from the genres list (appendix A) and
+ * ended with a ")" character. This is optionally followed by a
+ * refinement, e.g. "(21)" or "(4)Eurodisco". Several references can
+ * be made in the same frame, e.g. "(51)(39)". If the refinement
+ * should begin with a "(" character it should be replaced with "((",
+ * e.g. "((I can figure out any genre)" or "(55)((I think...)". The
+ * following new content types is defined in ID3v2 and is implemented
+ * in the same way as the numeric content types, e.g. "(RX)".
+ */
+
+    /* If a "refinement" exists, we will forget about the ID's. */
+    /* If only an ID is given, we will translate that ID into a string */
+    gchar *genre, *oldgenre, *utf8_genre=NULL;
+    const gchar *newgenre = NULL;
+    g_return_if_fail (genrep);
+
+    genre = *genrep;
+    oldgenre = *genrep;
+    if (genre == NULL) return;
+
+    while (*genre)
+    {
+	if (genre[0] == '(')
+	{
+	    if (genre[1] == '(')
+	    {
+		/* starting with "((" */
+		newgenre = &genre[1];
+		break;
+	    }
+	    if (isdigit (genre[1]))
+	    {   /* possibly a genre ID */
+		int num, genreid;
+		num = sscanf (genre, "(%d)", &genreid);
+		if (num != 1)
+		{   /* invalid ID -> give up */
+		    newgenre = &genre[0];
+		    break;
+		}
+		genre = strchr (&genre[1], ')');
+		g_return_if_fail (genre);
+		++genre;
+		if (!newgenre)
+		{   /* retrieve genre string from ID -- we only
+		     * convert the first ID */
+		    id3_ucs4_t const *ucs4_genre = id3_genre_index (genreid);
+		    if (ucs4_genre == NULL)
+		    {
+			break;
+		    }
+		    utf8_genre = id3_ucs4_utf8duplicate (ucs4_genre);
+		    newgenre = utf8_genre;
+		}
+	    }
+	    else
+	    {
+		newgenre = &genre[0];
+		break;
+	    }
+	}
+	else
+	{
+	    newgenre = &genre[0];
+	    break;
+	}
+    }
+    if (newgenre && (newgenre != oldgenre))
+    {
+	*genrep = g_strdup (newgenre);
+	g_free (oldgenre);
+    }
+    g_free (utf8_genre);
+}
+
+
+
+
 /***
  * Reads id3v1.x / id3v2 tag and load data into the Id3tag structure.
  * If a tag entry exists (ex: title), we allocate memory, else value
@@ -1459,11 +1558,13 @@ gboolean id3_tag_read (const gchar *filename, File_Tag *tag)
     if ((id3tag = id3_file_tag(id3file)))
     {
 	tag->title = id3_get_string (id3tag, ID3_FRAME_TITLE);
-	tag->artist = id3_get_string (id3tag, ID3_FRAME_GROUP);
+	tag->artist = id3_get_string (id3tag, ID3_FRAME_ARTIST);
 	if (!tag->artist || !*tag->artist)
 	{
 	    g_free (tag->artist);
-	    tag->artist = id3_get_string (id3tag, ID3_FRAME_ARTIST);
+	    tag->artist = id3_get_string (id3tag, ID3_FRAME_GROUP);
+	} else {
+	    tag->albumartist = id3_get_string (id3tag, ID3_FRAME_GROUP);
 	}
 	tag->album = id3_get_string (id3tag, ID3_FRAME_ALBUM);
 	tag->year = id3_get_string (id3tag, ID3_FRAME_YEAR);
@@ -1517,6 +1618,10 @@ gboolean id3_tag_read (const gchar *filename, File_Tag *tag)
 	    tag->cdnostring = g_strdup_printf ("%.2d", atoi (string));
 	    g_free(string);
 	}
+
+	/* Do some checks on the genre string -- ideally this should
+	 * be done within the id3tag library, I think */
+	handle_genre_variations (&tag->genre);
     }
 
     id3_file_close (id3file);
@@ -1572,9 +1677,9 @@ void set_uncommon_tag (struct id3_tag *id3tag,
 {
 #if 0
     struct id3_frame *frame;
+    union id3_field *field;
 
     frame = id3_tag_findframe (id3tag, id, 0);
-	union id3_field *field;
 	frame->flags = 0;
 	field = id3_frame_field (frame, 0);
 	    if (field)
@@ -1595,7 +1700,7 @@ void set_uncommon_tag (struct id3_tag *id3tag,
  * Write the ID3 tags to the file.
  * @returns: TRUE on success, else FALSE.
  */
-gboolean mp3_write_file_info (gchar *filename, Track *track)
+gboolean mp3_write_file_info (const gchar *filename, Track *track)
 {
     struct id3_tag* id3tag;
     struct id3_file* id3file;
@@ -1614,7 +1719,6 @@ gboolean mp3_write_file_info (gchar *filename, Track *track)
     if ((id3tag = id3_file_tag(id3file)))
     {
 	char *string1;
-
 	enum id3_field_textencoding encoding;
 
 	/* use the same coding as before... */
@@ -1637,6 +1741,7 @@ gboolean mp3_write_file_info (gchar *filename, Track *track)
 
 	id3_set_string (id3tag, ID3_FRAME_TITLE, track->title, encoding);
 	id3_set_string (id3tag, ID3_FRAME_ARTIST, track->artist, encoding);
+	id3_set_string (id3tag, ID3_FRAME_GROUP, track->albumartist, encoding);
 	id3_set_string (id3tag, ID3_FRAME_ALBUM, track->album, encoding);
 	id3_set_string (id3tag, ID3_FRAME_GENRE, track->genre, encoding);
 	id3_set_string (id3tag, ID3_FRAME_COMMENT, track->comment, encoding);
@@ -1912,14 +2017,16 @@ static inline guint16 parse_lame_uint16(char *buf) {
  * TODO: Check CRC.
  */
 
-gboolean mp3_get_track_lame_replaygain (gchar *path, GainData *gd)
+gboolean mp3_get_track_lame_replaygain (const gchar *path, GainData *gd)
 {
+	unsigned char ubuf[2];
+	int gain_adjust = 0;
+	LameTag lt;
+
 	g_return_val_if_fail (path, FALSE);
 
-	LameTag lt;
 	if (!mp3_read_lame_tag (path, &lt))
 	    goto rg_fail;
-	int gain_adjust = 0;
 
 	g_return_val_if_fail (gd, FALSE);
 
@@ -1963,7 +2070,6 @@ gboolean mp3_get_track_lame_replaygain (gchar *path, GainData *gd)
 				version[0], version[1], version[2], version[3], version[4]); */
 	}
 
-	unsigned char ubuf[2];
 	/* radio gain */
 	memcpy(ubuf,&lt.radio_replay_gain,2);
 	read_lame_replaygain (ubuf, gd, gain_adjust);
@@ -1988,7 +2094,7 @@ rg_fail:
  * The function only modifies the gains if they have not previously been set.
  */
 
-gboolean mp3_get_track_ape_replaygain(gchar *path, GainData *gd)
+gboolean mp3_get_track_ape_replaygain(const gchar *path, GainData *gd)
 {
 	/* The Ape Tag is located a t the end of the file. Or at least that
 	 * seems where it can most likely be found. Either it is at the very end
@@ -2167,7 +2273,7 @@ rg_fail:
  *
  * Returns TRUE if the soundcheck field could be set.
  */
-gboolean mp3_read_soundcheck (gchar *path, Track *track)
+gboolean mp3_read_soundcheck (const gchar *path, Track *track)
 {
     GainData gd;
 
@@ -2226,21 +2332,24 @@ int samplesperframe[2][3] = {
  * @path: location of the file
  * @lt: pointer to structure to be filled
  */
-gboolean mp3_read_lame_tag (gchar *path, LameTag *lt)
+gboolean mp3_read_lame_tag (const gchar *path, LameTag *lt)
 {
+    MP3Info *mp3i = NULL;
+    MP3Header h;
+    int flags;
+    int toskip = 0;
+    FILE *file;
     unsigned char ubuf[LAME_TAG_SIZE];
     int sideinfo;
-
     unsigned char full_info_tag[INFO_TAG_CRC_SIZE];
 
     g_return_val_if_fail (path, FALSE);
 
     /* Attempt to open the file */
-    FILE *file = fopen (path, "r");
+    file = fopen (path, "r");
     if (!file)
 	goto lt_fail;
 
-    MP3Info *mp3i = NULL;
     mp3i = g_malloc0 (sizeof (MP3Info));
     mp3i->filename = path;
     mp3i->file = file;
@@ -2253,7 +2362,6 @@ gboolean mp3_read_lame_tag (gchar *path, LameTag *lt)
 	goto lt_fail;
     fseek(mp3i->file, -INFO_TAG_CRC_SIZE, SEEK_CUR);
 
-    MP3Header h;
     if (!get_header (mp3i->file, &h))
 	goto lt_fail;
 
@@ -2282,22 +2390,20 @@ gboolean mp3_read_lame_tag (gchar *path, LameTag *lt)
 	goto lt_fail;
 
     /* Determine the offset of the LAME tag based on contents of the Xing header */
-    int flags;
     fread (&flags, 4, 1, mp3i->file);
-    int toskip = 0;
-    if (flags | 0x1)
+    if (flags & FRAMES_FLAG)
     {				/* frames field is set */
 	toskip += 4;
     }
-    if (flags | 0x2)
+    if (flags & BYTES_FLAG)
     {				/* bytes field is set */
 	toskip += 4;
     }
-    if (flags | 0x4)
+    if (flags & TOC_FLAG)
     {				/* TOC field is set */
 	toskip += 100;
     }
-    if (flags | 0x8)
+    if (flags & VBR_SCALE_FLAG)
     {				/* quality field is set */
 	toskip += 4;
     }
@@ -2346,11 +2452,13 @@ gboolean mp3_read_lame_tag (gchar *path, LameTag *lt)
     lt->calculated_info_tag_crc = crc_compute(full_info_tag, INFO_TAG_CRC_SIZE, 0x0000);
 
     fclose(file);
+    g_free (mp3i);
     return (lt->calculated_info_tag_crc == lt->info_tag_crc);
 
   lt_fail:
     if (file)
 	fclose(file);
+    g_free (mp3i);
     return FALSE;
 
 }
@@ -2368,6 +2476,13 @@ gboolean mp3_read_lame_tag (gchar *path, LameTag *lt)
 gboolean mp3_get_track_gapless (MP3Info *mp3i, GaplessData *gd)
 {
     int i;
+    int xing_header_offset;
+    int mysamplesperframe;
+    int totaldatasize;
+    int lastframes[8];
+    int totalframes;
+    int finaleight;
+    int l;
 
     g_return_val_if_fail (mp3i, FALSE);
     g_return_val_if_fail (gd, FALSE);
@@ -2375,27 +2490,26 @@ gboolean mp3_get_track_gapless (MP3Info *mp3i, GaplessData *gd)
     /* use get_first_header() to seek to the first mp3 header */
     get_first_header (mp3i, 0);
 
-    int xing_header_offset = ftell (mp3i->file);
+    xing_header_offset = ftell (mp3i->file);
 
     get_header(mp3i->file, &(mp3i->header));
 
-    int mysamplesperframe = samplesperframe[mp3i->header.version & 1][3 - mp3i->header.layer];
+    mysamplesperframe = samplesperframe[mp3i->header.version & 1][3 - mp3i->header.layer];
 
     /* jump to the end of the frame with the xing header */
     if (fseek (mp3i->file, xing_header_offset + frame_length (&(mp3i->header)), SEEK_SET))
 	goto gp_fail;
 
     /* counts bytes from the start of the 1st sync frame */
-    int totaldatasize = frame_length (&(mp3i->header));
+    totaldatasize = frame_length (&(mp3i->header));
 
-    /* keeps track of the last 8 frame sizes */
-    int lastframes[8];
+    /* lastframes keeps track of the last 8 frame sizes */
 
     /* counts number of music frames */
-    int totalframes = 0;
+    totalframes = 0;
 
     /* quickly parse the file, reading only frame headers */
-    int l = 0;
+    l = 0;
     while ((l = get_header (mp3i->file, &(mp3i->header))) != 0)
     {
 	lastframes[totalframes%8] = l;
@@ -2406,7 +2520,7 @@ gboolean mp3_get_track_gapless (MP3Info *mp3i, GaplessData *gd)
 	    goto gp_fail;
     }
 
-    int finaleight = 0;
+    finaleight = 0;
     for (i = 0; i < 8; i++)
     {
 	finaleight += lastframes[i];
@@ -2451,7 +2565,7 @@ gboolean mp3_get_track_gapless (MP3Info *mp3i, GaplessData *gd)
  * FALSE otherwise.
  */
 
-gboolean mp3_read_gapless (gchar *path, Track *track) {
+gboolean mp3_read_gapless (const gchar *path, Track *track) {
     MP3Info *mp3i=NULL;
     FILE *file;
 
@@ -2462,6 +2576,8 @@ gboolean mp3_read_gapless (gchar *path, Track *track) {
 
     etr = track->userdata;
 
+    g_return_val_if_fail (etr, FALSE);
+
     memset (&gd, 0, sizeof (GaplessData));
 
 
@@ -2484,6 +2600,7 @@ gboolean mp3_read_gapless (gchar *path, Track *track) {
 	} else {
 	    /* insert non-LAME methods of finding pregap and postgap */
 	    fclose(file);
+	    g_free (mp3i);
 	    return FALSE;
 	}
 	    
@@ -2493,21 +2610,36 @@ gboolean mp3_read_gapless (gchar *path, Track *track) {
 
 	if ((gd.pregap) && (gd.samplecount) && (gd.postgap) && (gd.gapless_data))
 	{
-	    if ((track->pregap != lt.delay) ||
+	    if ((track->pregap != gd.pregap) ||
 		(track->samplecount != gd.samplecount) ||
-		(track->postgap != lt.padding) ||
+		(track->postgap != gd.postgap) ||
 		(track->gapless_data != gd.gapless_data) ||
 		(track->gapless_track_flag == FALSE))
 	    {
 		etr->tchanged = TRUE;
-		track->pregap = lt.delay;
+		track->pregap = gd.pregap;
 		track->samplecount = gd.samplecount;
-		track->postgap = lt.padding;
+		track->postgap = gd.postgap;
 		track->gapless_data = gd.gapless_data;
 		track->gapless_track_flag = TRUE;
 	    }
 	}
+	else
+	{   /* remove gapless data which doesn't seem to be valid any
+	     * more */
+	    if (track->gapless_track_flag == TRUE)
+	    {
+		etr->tchanged = TRUE;
+	    }
+	    track->pregap = 0;
+	    track->samplecount = 0;
+	    track->postgap = 0;
+	    track->gapless_data = 0;
+	    track->gapless_track_flag = FALSE;
+	}
+	    
 	fclose(file);
+	g_free (mp3i);
 	return TRUE;
     }
     return FALSE;
@@ -2538,6 +2670,11 @@ gboolean id3_read_tags (const gchar *name, Track *track)
 	    track->artist = filetag.artist;
 	}
 
+	if (filetag.albumartist)
+	{
+	    track->albumartist = filetag.albumartist;
+	}
+
 	if (filetag.title)
 	{
 	    track->title = filetag.title;
@@ -2708,7 +2845,7 @@ gboolean id3_read_tags (const gchar *name, Track *track)
 
 /* Return a Track structure with all information read from the mp3
    file filled in */
-Track *mp3_get_file_info (gchar *name)
+Track *mp3_get_file_info (const gchar *name)
 {
     Track *track = NULL;
     MP3Info *mp3i=NULL;
@@ -2774,3 +2911,92 @@ Track *mp3_get_file_info (gchar *name)
     }
     return track;
 }
+/*
+ *
+ * @returns: TRUE on success, else FALSE.
+ */
+gboolean id3_lyrics_read (const gchar *filename,gchar **lyrics)
+{
+    struct id3_file *id3file;
+    struct id3_tag *id3tag;
+
+    g_return_val_if_fail (filename, FALSE);
+    g_return_val_if_fail (lyrics, FALSE);
+
+    if (!(id3file = id3_file_open (filename, ID3_FILE_MODE_READONLY)))
+    {
+	gchar *fbuf = charset_to_utf8 (filename);
+	g_print(_("ERROR while opening file: '%s' (%s).\n"),
+		fbuf, g_strerror(errno));
+	g_free (fbuf);
+	return FALSE;
+    }
+
+    if ((id3tag = id3_file_tag(id3file)))
+    {
+	*lyrics = id3_get_string (id3tag, "USLT");
+    }
+
+    id3_file_close (id3file);
+    return TRUE;
+}
+
+gboolean id3_lyrics_save (const gchar *filename,const gchar *lyrics)
+{
+    struct id3_file *id3file;
+    struct id3_tag *id3tag;
+
+    g_return_val_if_fail (filename, FALSE);
+
+
+    id3file = id3_file_open (filename, ID3_FILE_MODE_READWRITE);
+    if (!id3file)
+    {
+	gchar *fbuf = charset_to_utf8 (filename);
+	g_print(_("ERROR while opening file: '%s' (%s).\n"),
+		fbuf, g_strerror(errno));
+	g_free (fbuf);
+	return FALSE;
+    }
+
+    if ((id3tag = id3_file_tag(id3file)))
+    {
+	enum id3_field_textencoding encoding;
+
+	/* actually the iPod only understands UTF8 lyrics, but for
+	   consistency's sake we'll do the same as for the other tags
+	*/
+	/* use the same coding as before... */
+	encoding = get_encoding (id3tag);
+	/* ...unless it's ISO_8859_1 and prefs say we should use
+	   unicode (i.e. ID3v2.4) */
+	if (prefs_get_int("id3_write_id3v24") &&
+	    (encoding == ID3_FIELD_TEXTENCODING_ISO_8859_1))
+	    encoding = ID3_FIELD_TEXTENCODING_UTF_8;
+
+	/* always render id3v1 to prevent dj studio from crashing */
+	id3_tag_options(id3tag, ID3_TAG_OPTION_ID3V1, ~0);
+
+        /* turn off frame compression and crc information to let
+	   itunes read tags see
+	   http://www.mars.org/mailman/public/mad-dev/2002-October/000742.html
+	*/
+	id3_tag_options(id3tag, ID3_TAG_OPTION_COMPRESSION, 0);
+	id3_tag_options(id3tag, ID3_TAG_OPTION_CRC, 0);
+
+	id3_set_string (id3tag, "USLT", lyrics, encoding);
+    }
+
+    if (id3_file_update(id3file) != 0)
+    {
+	gchar *fbuf = charset_to_utf8 (filename);
+	g_print(_("ERROR while writing tag to file: '%s' (%s).\n"),
+		fbuf, g_strerror(errno));
+	g_free (fbuf);
+	return FALSE;
+    }
+
+    id3_file_close (id3file);
+
+    return TRUE;
+}
