@@ -1,4 +1,4 @@
-/* Time-stamp: <2007-06-23 02:49:08 jcs>
+/* Time-stamp: <2008-09-07 11:16:18 jcs>
 |
 |  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
@@ -24,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: mp4file.c 1575 2007-06-22 17:49:43Z jcsjcs $
+|  $Id: mp4file.c 2206 2009-01-11 13:00:36Z jcsjcs $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -137,7 +137,25 @@
 #include <inttypes.h>
 #include <stdlib.h>
 #include <string.h>
-#include "mp4.h"
+#ifdef HAVE_LIBMP4V2_2
+#include <mp4v2/mp4v2.h>
+#else
+#include <mp4.h>
+#endif
+
+#ifndef FREEFORM_ACCEPTS_EXTRA_ARG
+/* Version 1.6 of libmp4v2 introduces an index argument for MP4GetMetadataFreeForm. For C++ sources it defaults
+   to 0, but in C we have to specify it on our own. 
+*/
+#define MP4GetMetadataFreeForm(mp4File, name, pValue, pValueSize, owner)  MP4GetMetadataFreeForm(mp4File, name, pValue, pValueSize)
+#endif
+
+#ifndef COVERART_ACCEPTS_EXTRA_ARG
+/* Version 1.6 of libmp4v2 introduces an index argument for MP4GetMetadataCoverart. For C++ sources it defaults
+   to NULL, but in C we have to specify it on our own. 
+*/
+#define MP4GetMetadataCoverArt(hFile, coverArt, size, index) MP4GetMetadataCoverArt(hFile, coverArt, size)
+#endif
 
 static gboolean mp4_scan_soundcheck (MP4FileHandle mp4File, Track *track)
 {
@@ -149,7 +167,7 @@ static gboolean mp4_scan_soundcheck (MP4FileHandle mp4File, Track *track)
     g_return_val_if_fail (mp4File != MP4_INVALID_FILE_HANDLE, FALSE);
 
     if (MP4GetMetadataFreeForm(mp4File, "iTunNORM",
-			       &ppValue, &pValueSize))
+			       &ppValue, &pValueSize, NULL))
     {
 	gchar *str;
 	guint sc1=0, sc2=0;
@@ -171,7 +189,7 @@ static gboolean mp4_scan_soundcheck (MP4FileHandle mp4File, Track *track)
     }
 
     if (MP4GetMetadataFreeForm(mp4File, "replaygain_track_gain",
-			       &ppValue, &pValueSize))
+			       &ppValue, &pValueSize, NULL))
     {
 	gchar *str;
 	gdouble rg;
@@ -269,6 +287,53 @@ Track *mp4_get_file_info (gchar *mp4FileName)
 	{
 	    trackId = MP4FindTrackId(mp4File, track_cur, NULL, 0);
 	    trackType = MP4GetTrackType(mp4File, trackId);
+	    if (trackType && (strcmp(trackType, "text") == 0))
+	    {
+		u_int32_t m_max_frame_size;
+		m_max_frame_size = MP4GetTrackMaxSampleSize(mp4File, trackId) + 4;
+		MP4SampleId samples = MP4GetTrackNumberOfSamples(mp4File, trackId);
+		MP4SampleId i;
+		Itdb_Chapterdata *chapterdata = itdb_chapterdata_new();
+		for (i=1; i<=samples; i++)
+		{
+		    u_int8_t *m_buffer;
+		    m_buffer = (u_int8_t *) malloc(m_max_frame_size * sizeof(u_int8_t));
+		    u_int32_t m_this_frame_size = m_max_frame_size;
+		    u_int8_t *buffer;
+		    buffer = m_buffer;
+		    gchar *title;
+		    if (!MP4ReadSample(mp4File, trackId, i, &buffer, &m_this_frame_size, NULL, NULL, NULL, NULL))
+		    {
+			/* chapter title couldn't be read; probably using
+			 * an older version of libmp4v2.  We'll just make
+			 * our own titles, since the ipod doesn't display
+			 * them anyway
+			 */
+			free (m_buffer);
+			m_buffer = (u_int8_t *) malloc(12 * sizeof(u_int8_t));
+			sprintf(m_buffer, "Chapter %03i", i);
+			m_buffer[11] = '\0';
+			title = g_strdup(m_buffer);
+		    }
+		    else
+		    {
+			int titlelength = (buffer[0] << 8) + buffer[1];
+			gchar *newtitle = (gchar *) malloc((titlelength+1) * sizeof(gchar));
+			newtitle = g_strndup (&buffer[2], titlelength);
+			newtitle[titlelength] = '\0';
+			title = g_strdup (newtitle);
+			free (newtitle);
+		    }
+
+		    MP4Timestamp sampletime = MP4GetSampleTime(mp4File, trackId, i);
+		    u_int64_t convertedsampletime = MP4ConvertFromTrackTimestamp(mp4File,
+			    trackId, sampletime, MP4_MILLISECONDS_TIME_SCALE);
+		    itdb_chapterdata_add_chapter(chapterdata, convertedsampletime, title);
+		}
+		track->chapterdata = itdb_chapterdata_duplicate (chapterdata);
+
+		itdb_chapterdata_free(chapterdata);
+	    }
 	    if (trackType &&
 		(audio_or_video_found == FALSE) &&
 		((strcmp(trackType, MP4_AUDIO_TRACK_TYPE) == 0) ||
@@ -314,7 +379,31 @@ Track *mp4_get_file_info (gchar *mp4FileName)
 			track->artist = charset_to_utf8 (value);
 			g_free(value);
 		    }
+#if MP4_ALBUMARTIST_EXISTS
+		    if (!track->artist || !*track->artist)
+		    {
+			g_free (track->artist);
+			track->artist = NULL;
+			if (MP4GetMetadataAlbumArtist(mp4File, &value) && value != NULL)
+			{
+			    track->artist = charset_to_utf8 (value);
+			}
+		    }
+		    else
+		    {
+			if (MP4GetMetadataAlbumArtist(mp4File, &value) && value != NULL)
+			{
+			    track->albumartist = charset_to_utf8 (value);
+			}
+		    }
+#else
+#warning "Album Artist field not supported with this version of libmp4v2. Album Artist support requires at least V1.6.0"
+#endif
+#if HAVE_LIBMP4V2_2
+		    if (MP4GetMetadataComposer(mp4File, &value) && value != NULL)
+#else
 		    if (MP4GetMetadataWriter(mp4File, &value) && value != NULL)
+#endif
 		    {
 			track->composer = charset_to_utf8 (value);
 			g_free(value);
@@ -324,7 +413,11 @@ Track *mp4_get_file_info (gchar *mp4FileName)
 			track->comment = charset_to_utf8 (value);
 			g_free(value);
 		    }
+#if HAVE_LIBMP4V2_2
+		    if (MP4GetMetadataReleaseDate(mp4File, &value) && value != NULL)
+#else
 		    if (MP4GetMetadataYear(mp4File, &value) && value != NULL)
+#endif
 		    {
 			track->year = atoi (value);
 			g_free(value);
@@ -354,10 +447,18 @@ Track *mp4_get_file_info (gchar *mp4FileName)
 			track->genre = charset_to_utf8 (value);
 			g_free(value);
 		    }
+#if HAVE_LIBMP4V2_2
+		    if (MP4GetMetadataBPM (mp4File, &numvalue))
+#else
 		    if (MP4GetMetadataTempo (mp4File, &numvalue))
+#endif
 		    {
 			track->BPM = numvalue;
 		    }
+		    if (MP4HaveAtom (mp4File, "moov.udta.meta.ilst.\251lyr"))
+		    {
+			track->lyrics_flag = 0x01;
+		    }
 		}
 		mp4_scan_soundcheck (mp4File, track);
 		audio_or_video_found = TRUE;
@@ -367,7 +468,7 @@ Track *mp4_get_file_info (gchar *mp4FileName)
 		    u_int8_t *image_data;
 		    u_int32_t image_data_len;
 		    if (MP4GetMetadataCoverArt (mp4File,
-						&image_data, &image_data_len))
+						&image_data, &image_data_len, 0))
 		    {
 			if (image_data)
 			{
@@ -427,7 +528,7 @@ gboolean mp4_write_file_info (gchar *mp4FileName, Track *track)
 	     * you have to delete all meta data before modifying
 	     * it. Therefore we have to read it first to avoid data
 	     * loss. (Bug present in mpeg4ip-1.0RC1.) */
-/*	    gchar *m_name = NULL, *m_artist = NULL;
+/*	    gchar *m_name = NULL, *m_artist = NULL, *m_albumartist = NULL;
 	    gchar *m_writer = NULL, *m_comment = NULL;
 	    gchar *m_year = NULL;
 	    gchar *m_album = NULL, *m_genre = NULL;*/
@@ -440,19 +541,33 @@ gboolean mp4_write_file_info (gchar *mp4FileName, Track *track)
 						      &m_track, &m_tracks);
 	    gboolean has_disk = MP4GetMetadataDisk (mp4File,
 	    &m_disk, &m_disks);*/
+#if HAVE_LIBMP4V2_2
+	    gboolean has_tempo = MP4GetMetadataBPM (mp4File,
+						      &m_tempo);
+#else
 	    gboolean has_tempo = MP4GetMetadataTempo (mp4File,
 						      &m_tempo);
+#endif
 	    gboolean has_compilation = MP4GetMetadataCompilation (mp4File,
 								  &m_cpl);
 /*	    MP4GetMetadataName (mp4File, &m_name);
 	    MP4GetMetadataArtist (mp4File, &m_artist);
+	    MP4GetMetadataAlbumArtist (mp4File, &m_albumartist);
+#if HAVE_LIBMP4V2_2
+	    MP4GetMetadataComposer (mp4File, &m_writer);
+#else
 	    MP4GetMetadataWriter (mp4File, &m_writer);
+#endif
 	    MP4GetMetadataComment (mp4File, &m_comment);
+#if HAVE_LIBMP4V2_2
+	    MP4GetMetadataReleaseDate (mp4File, &m_year);
+#else
 	    MP4GetMetadataYear (mp4File, &m_year);
+#endif
 	    MP4GetMetadataAlbum (mp4File, &m_album);
 	    MP4GetMetadataGenre (mp4File, &m_genre);*/
 	    MP4GetMetadataTool (mp4File, &m_tool);
-	    MP4GetMetadataCoverArt (mp4File, &m_covert, &m_size);
+	    MP4GetMetadataCoverArt (mp4File, &m_covert, &m_size, 0);
 	    MP4MetadataDelete (mp4File);
 #endif
 	    value = charset_from_utf8 (track->title);
@@ -463,8 +578,17 @@ gboolean mp4_write_file_info (gchar *mp4FileName, Track *track)
 	    MP4SetMetadataArtist (mp4File, value);
 	    g_free (value);
 
+#if MP4_ALBUMARTIST_EXISTS
+	    value = charset_from_utf8 (track->albumartist);
+	    MP4SetMetadataAlbumArtist (mp4File, value);
+	    g_free (value);
+#endif
 	    value = charset_from_utf8 (track->composer);
+#if HAVE_LIBMP4V2_2
+	    MP4SetMetadataComposer (mp4File, value);
+#else
 	    MP4SetMetadataWriter (mp4File, value);
+#endif
 	    g_free (value);
 
 	    value = charset_from_utf8 (track->comment);
@@ -472,7 +596,11 @@ gboolean mp4_write_file_info (gchar *mp4FileName, Track *track)
 	    g_free (value);
 
 	    value = g_strdup_printf ("%d", track->year);
+#if HAVE_LIBMP4V2_2
+	    MP4SetMetadataReleaseDate (mp4File, value);
+#else
 	    MP4SetMetadataYear (mp4File, value);
+#endif
 	    g_free (value);
 
 	    value = charset_from_utf8 (track->album);
@@ -483,7 +611,11 @@ gboolean mp4_write_file_info (gchar *mp4FileName, Track *track)
 
 	    MP4SetMetadataDisk (mp4File, track->cd_nr, track->cds);
 
+#if HAVE_LIBMP4V2_2
+	    MP4SetMetadataBPM (mp4File, track->BPM);
+#else
 	    MP4SetMetadataTempo (mp4File, track->BPM);
+#endif
 
 	    value = charset_from_utf8 (track->grouping);
 	    MP4SetMetadataGrouping (mp4File, value);
@@ -494,12 +626,17 @@ gboolean mp4_write_file_info (gchar *mp4FileName, Track *track)
 	    g_free (value);
 
 #if MP4V2_HAS_METADATA_BUG
+#if HAVE_LIBMP4V2_2
+	    if (has_tempo) MP4SetMetadataBPM (mp4File, m_tempo);
+#else
 	    if (has_tempo) MP4SetMetadataTempo (mp4File, m_tempo);
+#endif
 	    if (has_compilation) MP4SetMetadataCompilation (mp4File, m_cpl);
 	    if (m_tool)     MP4SetMetadataTool (mp4File, m_tool);
 	    if (m_covert)   MP4SetMetadataCoverArt (mp4File, m_covert, m_size);
 /*	    g_free (m_name);
 	    g_free (m_artist);
+	    g_free (m_albumartist);
 	    g_free (m_writer);
 	    g_free (m_comment);
 	    g_free (m_year);
